# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import
import os

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import git

# Globals
git.__salt__ = {}
git.__grains__ = {}
git.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GitTestCase(TestCase):
    '''
        Validate the git state
    '''
    def test_latest(self):
        '''
            Test to make sure the repository is cloned and is up to date
        '''
        arg = ["git@gitlab.example.com:user/website.git"]
        ret = {'changes': {'new': 'git@gitlab.example.com:user/website.git',
                           'revision': None},
               'comment': 'Repository git@gitlab.example.'
               'com:user/website.git cloned to salt',
               'name': 'git@gitlab.example.com:user/website.git',
               'result': True}

        mock = MagicMock(return_value={'result': False, 'comment': '"rev"'
                                       'is not compatible with the "mirror"'
                                       'and "bare" arguments'})
        with patch.object(git, '_fail', mock):
            self.assertDictEqual(git.latest("git@gitlab.example."
                                            "com:user/website.git",
                                            True,
                                            mirror=True,
                                            bare=True),
                                 {'comment': '"rev"is not compatible with the'
                                  ' "mirror"and "bare" arguments',
                                  'result': False})

        mock = MagicMock(return_value={'result': False,
                                       'comment': '"target" option'
                                       ' is required'})
        with patch.object(git, '_fail', mock):
            self.assertDictEqual(git.latest("git@gitlab.example.com:"
                                            "user/website.git"),
                                 {'comment': '"target" option is required',
                                  'result': False})

        with patch.dict(git.__grains__, {"shell": True}):
            mock = MagicMock(return_value={'comment': 'onlyif execution'
                                           ' failed', 'skip_watch': True,
                                           'result': True})
            with patch.object(git, 'mod_run_check', mock):
                self.assertDictEqual(git.latest("git@gitlab.example.com:"
                                                "user/website.git",
                                                target="/usr/share/nginx/prod",
                                                onlyif=True),
                                     {'changes': {},
                                      'comment': 'onlyif execution failed',
                                      'name': 'git@gitlab.example.com:'
                                      'user/website.git',
                                      'result': True,
                                      'skip_watch': True})

            mock = MagicMock(return_value="salt")
            with patch.object(git, 'mod_run_check', mock):
                mock = MagicMock(return_value=True)
                with patch.object(os.path, 'isdir', mock):
                    mock = MagicMock(return_value=Exception)
                    with patch.dict(git.__salt__, {'git.revision': mock}):
                        mock = MagicMock(return_value="salt")
                        with patch.object(git, '_fail', mock):
                            self.assertEqual(git.latest("git@gitl"
                                                        "ab.example.com:user"
                                                        "/website.git",
                                                        target="/usr/share/n"
                                                        "ginx/prod"),
                                             "salt")

                    mock = MagicMock(return_value="salt")
                    with patch.dict(git.__salt__, {'git.revision': mock}):
                        with patch.dict(git.__salt__,
                                        {'git.current_branch': mock}):
                            mock = MagicMock(return_value=None)
                            with patch.dict(git.__salt__,
                                            {'git.ls_remote': mock}):
                                with patch.dict(git.__opts__, {'test': True}):
                                    mock = MagicMock(return_value=["salt"])
                                    with patch.object(git,
                                                      '_neutral_test', mock):
                                        self.assertListEqual(git.latest(arg[0],
                                                                        None,
                                                                        "salt"),
                                                             ["salt"])

                                with patch.dict(git.__opts__, {'test': False}):
                                    mock = MagicMock(return_value=[arg[0]])
                                    with patch.dict(git.__salt__,
                                                    {'git.remote_get': mock}):
                                        mock = MagicMock(return_value=0)
                                        with patch.dict(git.__salt__,
                                                        {'cmd.retcode': mock}):
                                            sub_test_latest(self, arg)

                mock = MagicMock(return_value=False)
                with patch.object(os.path, 'isdir', mock):
                    mock = MagicMock(side_effect=[False, True])
                    with patch.object(os.path, 'isdir', mock):
                        mock = MagicMock(return_value=True)
                        with patch.object(os, 'listdir', mock):
                            mock = MagicMock(return_value=["salt"])
                            with patch.object(git, '_fail', mock):
                                self.assertListEqual(git.latest(arg[0], None,
                                                                "salt"),
                                                     ["salt"])

                    with patch.dict(git.__opts__, {'test': True}):
                        mock = MagicMock(return_value=["salt"])
                        with patch.object(git, '_neutral_test', mock):
                            self.assertListEqual(git.latest(arg[0], None,
                                                            "salt"), ["salt"])

                    with patch.dict(git.__opts__, {'test': False}):
                        mock = MagicMock(side_effect=[Exception, True])
                        with patch.dict(git.__salt__, {'git.clone': mock}):
                            mock = MagicMock(return_value=["salt"])
                            with patch.object(git, '_fail', mock):
                                self.assertListEqual(git.latest(arg[0], None,
                                                                "salt"),
                                                     ["salt"])

                            self.assertEqual(git.latest(arg[0], None, "salt",
                                                        bare=True), ret)

    def test_present(self):
        '''
            Test to make sure the repository is present
        '''
        arg = ["git@gitlab.example.com:user/website.git"]
        ret = [{'changes': {},
                'comment': '',
                'name': 'git@gitlab.example.com:user/website.git',
                'result': True},
               {'changes': {'new repository': 'git@gitlab.example'
                            '.com:user/website.git'},
                'comment': 'Initialized repository git@gitlab.'
                'example.com:user/website.git',
                'name': 'git@gitlab.example.com:user/website.git',
                'result': True}
               ]

        mock = MagicMock(return_value=True)
        with patch.object(os.path, 'isdir', mock):
            with patch.object(os.path, 'isfile', mock):
                self.assertDictEqual(git.present(arg[0], True), ret[0])

            self.assertDictEqual(git.present(arg[0], None), ret[0])

            with patch.object(os, 'listdir', mock):
                mock = MagicMock(return_value=["salt"])
                with patch.object(git, '_fail', mock):
                    self.assertListEqual(git.present(arg[0]), ["salt"])

        mock = MagicMock(return_value=False)
        with patch.object(os.path, 'isdir', mock):
            with patch.dict(git.__opts__, {'test': True}):
                mock = MagicMock(return_value="Dude")
                with patch.object(git, '_neutral_test', mock):
                    self.assertEqual(git.present(arg[0]), "Dude")

            with patch.dict(git.__opts__, {'test': False}):
                with patch.dict(git.__salt__, {'git.init': mock}):
                    self.assertDictEqual(git.present(arg[0]), ret[1])

    def test_config(self):
        '''
            Test to manage a git config setting
        '''
        arg = ["git@gitlab.example.com:user/website.git"]
        ret = [{'changes': {},
                'comment': 'No changes made',
                'name': 'git@gitlab.example.com:user/website.git',
                'result': True}
               ]
        mock = MagicMock(return_value=True)
        with patch.dict(git.__salt__, {'git.config_get': mock}):
            self.assertDictEqual(git.config(arg[0], True), ret[0])

    def test_mod_run_check(self):
        '''
            Test to execute the onlyif and unless logic.
        '''
        ret = [{'comment': 'onlyif execution failed',
                'result': True,
                'skip_watch': True},
               {'comment': 'unless execution succeeded',
                'result': True,
                'skip_watch': True}
               ]
        run_check_cmd_kwargs = {}
        run_check_cmd_kwargs['shell'] = "Salt"
        mock = MagicMock(side_effect=[1, 0])
        with patch.dict(git.__salt__, {'cmd.retcode': mock}):
            self.assertDictEqual(git.mod_run_check(run_check_cmd_kwargs,
                                                   True,
                                                   False),
                                 ret[0])

            self.assertDictEqual(git.mod_run_check(run_check_cmd_kwargs,
                                                   False,
                                                   True),
                                 ret[1])

        self.assertTrue(git.mod_run_check(run_check_cmd_kwargs, False, False))


def sub_test_latest(self, arg):
    '''
        Sub part of test_latest
    '''
    mock = MagicMock(return_value=0)
    with patch.dict(git.__salt__, {'git.checkout': mock}):
        mock = MagicMock(return_value=0)
        with patch.dict(git.__salt__, {'git.config_get': mock}):
            with patch.dict(git.__opts__, {'test': True}):
                mock = MagicMock(return_value="salt")
                with patch.object(git, '_neutral_test', mock):
                    self.assertEqual(git.latest(arg[0],
                                                True,
                                                "salt"),
                                     "salt")

if __name__ == '__main__':
    from integration import run_tests
    run_tests(GitTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import git
from salt.exceptions import SaltInvocationError

# Globals
git.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GitTestCase(TestCase):
    '''
    Test cases for salt.modules.git
    '''
    # 'get_' function tests: 1

    def test_current_branch(self):
        '''
        Test if it returns the current branch name
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(git.__salt__, {'cmd.run_stdout': mock}):
            self.assertTrue(git.current_branch('develop'))

    # 'revision' function tests: 1

    def test_revision(self):
        '''
        Test if it returns the long hash of a given identifier
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.revision('develop'),)

    # 'clone' function tests: 1

    def test_clone(self):
        '''
        Test if it clone a new repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.clone('origin', 'develop'))

    # 'describe' function tests: 1

    def test_describe(self):
        '''
        Test if it returns the git describe string (or the SHA hash
        if there are no tags) for the given revision
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(git.__salt__, {'cmd.run_stdout': mock}):
            self.assertTrue(git.describe('develop'))

    # 'archive' function tests: 1

    def test_archive(self):
        '''
        Test if it exports a tarball from the repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        mock_val = MagicMock(return_value='true')
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': mock_val}):
            self.assertTrue(git.archive('develop', 'archive.tar.gz'))

    # 'fetch' function tests: 1

    def test_fetch(self):
        '''
        Test if it perform a fetch on the given repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.fetch('develop'))

    # 'pull' function tests: 1

    def test_pull(self):
        '''
        Test if it perform a pull on the given repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.pull('develop'))

    # 'rebase' function tests: 1

    def test_rebase(self):
        '''
        Test if it rebase the current branch
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.rebase('develop'), True)

    # 'checkout' function tests: 1

    def test_checkout(self):
        '''
        Test if it checkout a given revision
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.checkout('develop', 'mybranch'))

    # 'merge' function tests: 1

    def test_merge(self):
        '''
        Test if it merge a given branch
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.merge('develop'))

    # 'init' function tests: 1

    def test_init(self):
        '''
        Test if it initialize a new git repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.init('develop'))

    # 'submodule' function tests: 1

    def test_submodule(self):
        '''
        Test if it initialize git submodules
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.submodule('develop'))

    # 'status' function tests: 1

    def test_status(self):
        '''
        Test if it return the status of the repository
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertListEqual(git.status('develop'), [])

    # 'add' function tests: 1

    def test_add(self):
        '''
        Test if it add a file to git
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.add('develop',
                                    '/salt/tests/unit/modules/example.py'))

    # 'rm' function tests: 1

    def test_rm(self):
        '''
        Test if it remove a file to git
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.rm('develop',
                                   '/salt/tests/unit/modules/example.py'))

    # 'commit' function tests: 1

    def test_commit(self):
        '''
        Test if it create a commit
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.commit('develop', 'The comit message'))

    # 'push' function tests: 1

    def test_push(self):
        '''
        Test if it Push to remote
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertTrue(git.push('develop', 'remote-name'))

    # 'remotes' function tests: 1

    def test_remotes(self):
        '''
        Test if it gets remotes like git remote -v
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertDictEqual(git.remotes('develop'), {})

    # 'remote_get' function tests: 1

    def test_remote_get(self):
        '''
        Test if it get the fetch and push URL for a specified remote name
        '''
        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': '\nSalt\nStack'})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.remote_get('develop'), ('Salt', 'Stack'))

        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': '\norigin\norigin'})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.remote_get('develop'), None)

        mock = MagicMock(return_value={'retcode': 1,
                                       'stdout': '\norigin\norigin',
                                       'stderr': 'error'})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.remote_get('develop'), None)

    # 'remote_set' function tests: 1

    def test_remote_set(self):
        '''
        Test if it sets a remote with name and URL like git remote add name url
        '''
        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': '\nSalt\nStack'})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.remote_set('develop'), ('Salt', 'Stack'))

    # 'branch' function tests: 1

    def test_branch(self):
        '''
        Test if it interacts with branches
        '''
        mock_all = MagicMock(return_value={'retcode': 0, 'stdout': ''})
        mock_stdout = MagicMock(return_value=True)
        with patch.dict(git.__salt__, {'cmd.run_all': mock_all,
                                       'cmd.run_stdout': mock_stdout}):
            self.assertEqual(git.branch('develop', 'origin/develop'), True)

    # 'reset' function tests: 1

    def test_reset(self):
        '''
        Test if it reset the repository checkout
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.reset('develop'), True)

    # 'stash' function tests: 1

    def test_stash(self):
        '''
        Test if stash changes in the repository checkout
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.stash('develop'), True)

    # 'config_set' function tests: 1

    def test_config_set(self):
        '''
        Test if it sets a key in the git configuration file
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertRaises(TypeError, git.config_set)

        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertRaises(SaltInvocationError, git.config_set,
                              None, 'myname', 'me@example.com')

        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.config_set(None, 'myname', 'me@example.com',
                                            'me', True), True)

    # 'config_get' function tests: 1

    def test_config_get(self):
        '''
        Test if it gets a key or keys from the git configuration file
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertRaises(TypeError, git.config_get)

        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.config_get(None, 'myname', 'me'), True)

    # 'ls_remote' function tests: 1

    def test_ls_remote(self):
        '''
        Test if it returns the upstream hash for any given URL and branch.
        '''
        mock = MagicMock(return_value={'retcode': 0, 'stdout': True})
        with patch.dict(git.__salt__, {'cmd.run_all': mock,
                                       'cmd.run_stdout': True}):
            self.assertEqual(git.ls_remote('develop'), True)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GitTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import rest_package

# Globals
rest_package.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RestPkgTestCase(TestCase):
    '''
    Test cases for salt.modules.rest_package
    '''
    def test_list_pkgs(self):
        '''
        Test for list pkgs
        '''
        with patch.dict(rest_package.__opts__, {'proxyobject': MagicMock()}):
            self.assertTrue(rest_package.list_pkgs())

    def test_install(self):
        '''
        Test for install
        '''
        with patch.dict(rest_package.__opts__, {'proxyobject': MagicMock()}):
            self.assertTrue(rest_package.install())

    def test_remove(self):
        '''
        Test for remove
        '''
        with patch.dict(rest_package.__opts__, {'proxyobject': MagicMock()}):
            self.assertTrue(rest_package.remove())

    def test_version(self):
        '''
        Test to return a string representing the package version or
        an empty string if not installed.
        '''
        with patch.dict(rest_package.__opts__, {'proxyobject': MagicMock()}):
            self.assertTrue(rest_package.version('A'))

    def test_installed(self):
        '''
        Test for installed
        '''
        with patch.dict(rest_package.__opts__, {'proxyobject': MagicMock()}):
            with patch.object(rest_package.__opts__['proxyobject'],
                              'package_status',
                              MagicMock(return_value={'ret': 'ret'})):
                self.assertEqual(rest_package.installed('name'), 'ret')

                self.assertTrue(rest_package.installed('name'))

                self.assertFalse(rest_package.installed('name', version='v'))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(RestPkgTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Module for managing Windows Users

:depends:
        - os
        - pywintypes
        - win32api
        - win32net
        - win32netcon
        - win32profile
        - win32security
        - win32ts

NOTE: This currently only works with local user accounts, not domain accounts
'''
from __future__ import absolute_import

# Import salt libs
import salt.utils
from salt.ext.six import string_types
from salt.exceptions import CommandExecutionError
import logging

log = logging.getLogger(__name__)

try:
    import os
    import pywintypes
    import win32api
    import win32net
    import win32netcon
    import win32profile
    import win32security
    import win32ts
    HAS_WIN32NET_MODS = True
except ImportError:
    HAS_WIN32NET_MODS = False

# Define the module's virtual name
__virtualname__ = 'user'


def __virtual__():
    '''
    Set the user module if the kernel is Windows
    '''
    if HAS_WIN32NET_MODS and salt.utils.is_windows():
        return __virtualname__
    return False


def add(name,
        password=None,
        fullname=False,
        description=None,
        groups=None,
        home=None,
        homedrive=None,
        profile=None,
        logonscript=None):
    '''
    Add a user to the minion.

    :param name: str
    User name

    :param password: str
    User's password in plain text.

    :param fullname: str
    The user's full name.

    :param description: str
    A brief description of the user account.

    :param groups: list
    A list of groups to add the user to.

    :param home: str
    The path to the user's home directory.

    :param homedrive: str
    The drive letter to assign to the home directory. Must be the Drive Letter
    followed by a colon. ie: U:

    :param profile: str
    An explicit path to a profile. Can be a UNC or a folder on the system. If
    left blank, windows uses it's default profile directory.

    :param logonscript: str
    Path to a login script to run when the user logs on.

    :return: bool
    True if successful. False is unsuccessful.

    CLI Example:

    .. code-block:: bash

        salt '*' user.add name password
    '''
    user_info = {}
    if name:
        user_info['name'] = name
    else:
        return False
    user_info['password'] = password
    user_info['priv'] = win32netcon.USER_PRIV_USER
    user_info['home_dir'] = home
    user_info['comment'] = description
    user_info['flags'] = win32netcon.UF_SCRIPT
    user_info['script_path'] = logonscript

    try:
        win32net.NetUserAdd(None, 1, user_info)
    except win32net.error as exc:
        (number, context, message) = exc
        log.error('Failed to create user {0}'.format(name))
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))
        return False

    update(name=name,
           homedrive=homedrive,
           profile=profile,
           fullname=fullname)

    if groups:
        ret = chgroups(name, groups)

    return ret


def update(name,
           password=None,
           fullname=None,
           description=None,
           home=None,
           homedrive=None,
           logonscript=None,
           profile=None):
    r'''
    Updates settings for the windows user. Name is the only required parameter.
    Settings will only be changed if the parameter is passed a value.

    .. versionadded:: 2015.8.0

    :param name: str
    The user name to update.

    :param password: str
    New user password in plain text.

    :param fullname: str
    The user's full name.

    :param description: str
    A brief description of the user account.

    :param home: str
    The path to the user's home directory.

    :param homedrive: str
    The drive letter to assign to the home directory. Must be the Drive Letter
    followed by a colon. ie: U:

    :param logonscript: str
    The path to the logon script.

    :param profile: str
    The path to the user's profile directory.

    :return: bool
    True if successful. False is unsuccessful.

    CLI Example:

    .. code-block:: bash

        salt '*' user.update bob password=secret profile=C:\Users\Bob
                 home=\\server\homeshare\bob homedrive=U:
    '''

    # Make sure the user exists
    # Return an object containing current settings for the user
    try:
        user_info = win32net.NetUserGetInfo(None, name, 4)
    except win32net.error as exc:
        (number, context, message) = exc
        log.error('Failed to update user {0}'.format(name))
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))
        return False

    # Check parameters to update
    # Update the user object with new settings
    if password:
        user_info['password'] = password
    if home:
        user_info['home_dir'] = home
    if homedrive:
        user_info['home_dir_drive'] = homedrive
    if description:
        user_info['comment'] = description
    if logonscript:
        user_info['script_path'] = logonscript
    if fullname:
        user_info['full_name'] = fullname
    if profile:
        user_info['profile'] = profile

    # Apply new settings
    try:
        win32net.NetUserSetInfo(None, name, 4, user_info)
    except win32net.error as exc:
        (number, context, message) = exc
        log.error('Failed to update user {0}'.format(name))
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))
        return False

    return True


def delete(name,
           purge=False,
           force=False):
    '''
    Remove a user from the minion

    :param name:
    The name of the user to delete

    :param purge:
    Boolean value indicating that the user profile should also be removed when
    the user account is deleted. If set to True the profile will be removed.

    :param force:
    Boolean value indicating that the user account should be deleted even if the
    user is logged in. True will log the user out and delete user.

    CLI Example:

    .. code-block:: bash

        salt '*' user.delete name
    '''
    # Check if the user exists
    try:
        user_info = win32net.NetUserGetInfo(None, name, 4)
    except win32net.error as exc:
        (number, context, message) = exc
        log.error('User not found: {0}'.format(name))
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))
        return False

    # Check if the user is logged in
    # Return a list of logged in users
    try:
        sess_list = win32ts.WTSEnumerateSessions()
    except win32ts.error as exc:
        (number, context, message) = exc
        log.error('No logged in users found')
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))

    # Is the user one that is logged in
    logged_in = False
    session_id = None
    for sess in sess_list:
        if win32ts.WTSQuerySessionInformation(None, sess['SessionId'], win32ts.WTSUserName) == name:
            session_id = sess['SessionId']
            logged_in = True

    # If logged in and set to force, log the user out and continue
    # If logged in and not set to force, return false
    if logged_in:
        if force:
            try:
                win32ts.WTSLogoffSession(win32ts.WTS_CURRENT_SERVER_HANDLE, session_id, True)
            except win32ts.error as exc:
                (number, context, message) = exc
                log.error('User not found: {0}'.format(name))
                log.error('nbr: {0}'.format(number))
                log.error('ctx: {0}'.format(context))
                log.error('msg: {0}'.format(message))
                return False
        else:
            log.error('User {0} is currently logged in.'.format(name))
            return False

    # Remove the User Profile directory
    if purge:
        # If the profile is not defined, get the profile from the registry
        if user_info['profile'] == '':
            profiles_dir = __salt__['reg.read_key'](hkey='HKEY_LOCAL_MACHINE',
                                                    path='SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList',
                                                    key='ProfilesDirectory')
            profiles_dir = profiles_dir.replace('%SystemDrive%', os.environ['SystemDrive'])
            user_info['profile'] = r'{0}\{1}'.format(profiles_dir, name)

        # Make sure the profile exists before deleting it
        # Otherwise this will throw an error
        if os.path.exists(user_info['profile']):
            sid = getUserSid(name)
            try:
                win32profile.DeleteProfile(sid)
            except pywintypes.error as exc:
                (number, context, message) = exc
                log.error('Failed to remove profile for {0}'.format(name))
                log.error('nbr: {0}'.format(number))
                log.error('ctx: {0}'.format(context))
                log.error('msg: {0}'.format(message))
                return False

    # And finally remove the user account
    try:
        win32net.NetUserDel(None, name)
    except win32net.error as exc:
        (number, context, message) = exc
        log.error('Failed to delete user {0}'.format(name))
        log.error('nbr: {0}'.format(number))
        log.error('ctx: {0}'.format(context))
        log.error('msg: {0}'.format(message))
        return False

    return True


def getUserSid(username):
    domain = win32api.GetComputerName()
    if username.find(u'\\') != -1:
        domain = username.split(u'\\')[0]
        username = username.split(u'\\')[-1]
    domain = domain.upper()
    return win32security.ConvertSidToStringSid(win32security.LookupAccountName(None, domain + u'\\' + username)[0])


def setpassword(name, password):
    '''
    Set a user's password

    CLI Example:

    .. code-block:: bash

        salt '*' user.setpassword name password
    '''
    return update(name=name, password=password)


def addgroup(name, group):
    '''
    Add user to a group

    CLI Example:

    .. code-block:: bash

        salt '*' user.addgroup username groupname
    '''
    user = info(name)
    if not user:
        return False
    if group in user['groups']:
        return True
    ret = __salt__['cmd.run_all'](
        'net localgroup {0} {1} /add'.format(group, name)
    )
    return ret['retcode'] == 0


def removegroup(name, group):
    '''
    Remove user from a group

    CLI Example:

    .. code-block:: bash

        salt '*' user.removegroup username groupname
    '''
    user = info(name)

    if not user:
        return False

    if group not in user['groups']:
        return True

    ret = __salt__['cmd.run_all'](
        'net localgroup {0} {1} /delete'.format(group, name)
    )
    return ret['retcode'] == 0


def chhome(name, home, persist=False):
    '''
    Change the home directory of the user, pass True for persist to move files
    to the new home directory if the old home directory exist.

    CLI Example:

    .. code-block:: bash

        salt '*' user.chhome foo \\\\fileserver\\home\\foo True
    '''
    pre_info = info(name)

    if not pre_info:
        return False

    if home == pre_info['home']:
        return True

    if not update(name=name, home=home):
        return False

    if persist and home is not None and pre_info['home'] is not None:
        cmd = 'move /Y {0} {1}'.format(pre_info['home'], home)
        if __salt__['cmd.retcode'](cmd, python_shell=False) != 0:
            log.debug('Failed to move the contents of the Home Directory')

    post_info = info(name)
    if post_info['home'] != pre_info['home']:
        return post_info['home'] == home

    return False


def chprofile(name, profile):
    '''
    Change the profile directory of the user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chprofile foo \\\\fileserver\\profiles\\foo
    '''
    return update(name=name, profile=profile)


def chfullname(name, fullname):
    '''
    Change the full name of the user

    CLI Example:

    .. code-block:: bash

        salt '*' user.chfullname user 'First Last'
    '''
    return update(name=name, fullname=fullname)


def chgroups(name, groups, append=False):
    '''
    Change the groups this user belongs to, add append to append the specified
    groups

    CLI Example:

    .. code-block:: bash

        salt '*' user.chgroups foo wheel,root True
    '''
    if isinstance(groups, string_types):
        groups = groups.split(',')

    groups = [x.strip(' *') for x in groups]
    ugrps = set(list_groups(name))
    if ugrps == set(groups):
        return True

    if not append:
        for group in ugrps:
            if group not in groups:
                __salt__['cmd.retcode'](
                        'net localgroup {0} {1} /delete'.format(group, name))

    for group in groups:
        if group in ugrps:
            continue
        __salt__['cmd.retcode'](
                'net localgroup {0} {1} /add'.format(group, name))
    agrps = set(list_groups(name))
    return len(ugrps - agrps) == 0


def info(name):
    '''
    Return user information

    :param name: str
    Username for which to display information

    :returns: dict
    A dictionary containing user information
    - fullname
    - username
    - uid
    - passwd (will always return None)
    - comment (same as description, left here for backwards compatibility)
    - description
    - active
    - logonscript
    - profile
    - home
    - homedrive
    - groups
    - gid

    CLI Example:

    .. code-block:: bash

        salt '*' user.info root
    '''
    ret = {}
    items = {}
    try:
        items = win32net.NetUserGetInfo(None, name, 4)
    except win32net.error:
        pass

    if items:
        groups = []
        try:
            groups = win32net.NetUserGetLocalGroups(None, name)
        except win32net.error:
            pass

        ret['fullname'] = items['full_name']
        ret['name'] = items['name']
        ret['uid'] = win32security.ConvertSidToStringSid(items['user_sid'])
        ret['passwd'] = items['password']
        ret['comment'] = items['comment']
        ret['description'] = items['comment']
        ret['active'] = (not bool(items['flags'] & win32netcon.UF_ACCOUNTDISABLE))
        ret['logonscript'] = items['script_path']
        ret['profile'] = items['profile']
        if not ret['profile']:
            ret['profile'] = _get_userprofile_from_registry(name, ret['uid'])
        ret['home'] = items['home_dir']
        ret['homedrive'] = items['home_dir_drive']
        if not ret['home']:
            ret['home'] = ret['profile']
        ret['groups'] = groups
        ret['gid'] = ''

    return ret


def _get_userprofile_from_registry(user, sid):
    '''
    In case net user doesn't return the userprofile
    we can get it from the registry
    '''
    profile_dir = __salt__['reg.read_key'](
        'HKEY_LOCAL_MACHINE', u'SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\{0}'.format(sid),
        'ProfileImagePath'
    )
    log.debug(u'user {0} with sid={2} profile is located at "{1}"'.format(user, profile_dir, sid))
    return profile_dir


def list_groups(name):
    '''
    Return a list of groups the named user belongs to

    CLI Example:

    .. code-block:: bash

        salt '*' user.list_groups foo
    '''
    ugrp = set()
    try:
        user = info(name)['groups']
    except KeyError:
        return False
    for group in user:
        ugrp.add(group.strip(' *'))

    return sorted(list(ugrp))


def getent(refresh=False):
    '''
    Return the list of all info for all users

    CLI Example:

    .. code-block:: bash

        salt '*' user.getent
    '''
    if 'user.getent' in __context__ and not refresh:
        return __context__['user.getent']

    ret = []
    for user in __salt__['user.list_users']():
        stuff = {}
        user_info = __salt__['user.info'](user)

        stuff['gid'] = ''
        stuff['groups'] = user_info['groups']
        stuff['home'] = user_info['home']
        stuff['name'] = user_info['name']
        stuff['passwd'] = user_info['passwd']
        stuff['shell'] = ''
        stuff['uid'] = user_info['uid']

        ret.append(stuff)

    __context__['user.getent'] = ret
    return ret


def list_users():
    '''
    Return a list of users on Windows
    '''
    res = 0
    users = []
    user_list = []
    dowhile = True
    try:
        while res or dowhile:
            dowhile = False
            (users, _, res) = win32net.NetUserEnum(
                None,
                0,
                win32netcon.FILTER_NORMAL_ACCOUNT,
                res,
                win32netcon.MAX_PREFERRED_LENGTH
            )
            for user in users:
                user_list.append(user['name'])
        return user_list
    except win32net.error:
        pass


def rename(name, new_name):
    '''
    Change the username for a named user

    CLI Example:

    .. code-block:: bash

        salt '*' user.rename name new_name
    '''
    current_info = info(name)
    if not current_info:
        raise CommandExecutionError('User {0!r} does not exist'.format(name))
    new_info = info(new_name)
    if new_info:
        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
    cmd = 'wmic useraccount where name="{0}" rename {1}'.format(name, new_name)
    __salt__['cmd.run'](cmd)
    post_info = info(new_name)
    if post_info['name'] != current_info['name']:
        return post_info['name'] == new_name
    return False



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import junos

# Globals
junos.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JunosTestCase(TestCase):
    '''
    Test cases for salt.modules.junos
    '''
    def test_facts_refresh(self):
        '''
        Test for Reload the facts dictionary from the device
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertTrue(junos.facts_refresh())

    def test_set_hostname(self):
        '''
        Test for set hostname
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertEqual(junos.set_hostname(), {'out': False})

            with patch.object(junos, 'commit', return_value=None):
                self.assertIsNone(junos.set_hostname('host', True))

            self.assertDictEqual(junos.set_hostname('host', False),
                                 {'msg': 'set system host-name host is queued',
                                  'out': True})

    def test_commit(self):
        '''
        Test for commit
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertDictEqual(junos.commit(),
                                 {'message': 'Commit Successful.',
                                  'out': True})

    def test_rollback(self):
        '''
        Test for rollback
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertEqual(junos.rollback()['message'],
                             'Rollback successful')

    def test_diff(self):
        '''
        Test for diff
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertTrue(junos.diff()['out'])

    def test_ping(self):
        '''
        Test for ping
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(junos.__opts__, {'proxyobject': mock}):
            self.assertIsNone(junos.ping())

if __name__ == '__main__':
    from integration import run_tests
    run_tests(JunosTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
unit tests for the grains state
'''

from __future__ import absolute_import

# Import Python libs
import os

# Import Salt Testing libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import NO_MOCK, NO_MOCK_REASON, MagicMock

ensure_in_syspath('../../')

from salt.modules import grains as grainsmod
from salt.states import grains as grains

import integration


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GrainsTestCase(TestCase):

    # 'present' function tests: 4

    def setUp(self):
        grains_test_dir = '__salt_test_state_grains'
        grainsmod.__opts__ = grains.__opts__ = {
            'test': False,
            'conf_file': os.path.join(integration.TMP, grains_test_dir, 'minion'),
            'cachedir':  os.path.join(integration.TMP, grains_test_dir),
            'local': True,
        }

        grainsmod.__salt__ = grains.__salt__ = {
            'cmd.run_all': MagicMock(return_value={
                'pid': 5,
                'retcode': 0,
                'stderr': '',
                'stdout': ''}),
            'grains.setval': grainsmod.setval,
            'grains.delval': grainsmod.delval,
            'grains.append': grainsmod.append,
            'grains.remove': grainsmod.remove,
            'saltutil.sync_grains': MagicMock()
        }
        if not os.path.exists(os.path.join(integration.TMP, grains_test_dir)):
            os.mkdir(os.path.join(integration.TMP, grains_test_dir))

    def test_present_add(self):
        # Set a non existing grain
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval'}
        ret = grains.present(
            name='foo',
            value='bar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['changes'], {'foo': 'bar'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

    def test_present_already_set(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        # Grain already set
        ret = grains.present(
            name='foo',
            value='bar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Grain is already set')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

    def test_present_overwrite(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        # Overwrite an existing grain
        ret = grains.present(
            name='foo',
            value='newbar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['changes'], {'foo': 'newbar'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'newbar'})

        # Overwrite a grain to a list
        ret = grains.present(
            name='foo',
            value=['l1', 'l2'])
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['changes'], {'foo': ['l1', 'l2']})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['l1', 'l2']})

        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        # Fails setting a grain to a dict
        ret = grains.present(
            name='foo',
            value={'k1': 'v1'})
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Grain value cannot be dict')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        # Clear a grain (set to None)
        ret = grains.present(
            name='foo',
            value=None)
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['changes'], {'foo': None})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': None})

    def test_present_unknown_failure(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        # Unknown reason failure
        grainsmod.__salt__['grains.setval'] = MagicMock()
        ret = grains.present(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Failed to set grain foo')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

    # 'absent' function tests: 3

    def test_absent_already(self):
        # Unset a non existent grain
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval'}
        ret = grains.absent(
            name='foo')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Grain foo does not exist')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval'})

    def test_absent_unset(self):
        # Unset a grain
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        ret = grains.absent(
            name='foo')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value for grain foo was set to None')
        self.assertEqual(ret['changes'], {'grain': 'foo', 'value': None})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': None})

    def test_absent_delete(self):
        # Delete a grain
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        ret = grains.absent(
            name='foo',
            destructive=True)
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Grain foo was deleted')
        self.assertEqual(ret['changes'], {'deleted': 'foo'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval'})

    # 'append' function tests: 5

    def test_append(self):
        # Append to an existing list
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.append(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value baz was added to grain foo')
        self.assertEqual(ret['changes'], {'added': 'baz'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar', 'baz']})

    def test_append_already(self):
        # Append to an existing list
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.append(
            name='foo',
            value='bar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value bar is already in the list '
                                       + 'for grain foo')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar']})

    def test_append_fails_not_a_list(self):
        # Fail to append to an existing grain, not a list
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': {'bar': 'val'}}
        ret = grains.append(
            name='foo',
            value='baz')
        # Note from dr4Ke: should be false, IMO
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Grain foo is not a valid list')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': {'bar': 'val'}})

    def test_append_convert_to_list(self):
        # Append to an existing grain, converting to a list
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': {'bar': 'val'}}
        ret = grains.append(
            name='foo',
            value='baz',
            convert=True)
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value baz was added to grain foo')
        self.assertEqual(ret['changes'], {'added': 'baz'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': [{'bar': 'val'}, 'baz']})

    def test_append_fails_inexistent(self):
        # Append to a non existing grain
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval'}
        ret = grains.append(
            name='foo',
            value='bar')
        # Note from dr4Ke: should be false, IMO
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Grain foo does not exist')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval'})

    # 'list_present' function tests: 5

    def test_list_present(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.list_present(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Append value baz to grain foo')
        self.assertEqual(ret['changes'], {'new': {'foo': ['bar', 'baz']}})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar', 'baz']})

    def test_list_present_inexistent(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval'}
        ret = grains.list_present(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Append value baz to grain foo')
        self.assertEqual(ret['changes'], {'new': {'foo': ['baz']}})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['baz']})

    def test_list_present_not_a_list(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        ret = grains.list_present(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Grain foo is not a valid list')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

    def test_list_present_already(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.list_present(
            name='foo',
            value='bar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value bar is already in grain foo')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar']})

    def test_list_present_unknown_failure(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        # Unknown reason failure
        grainsmod.__salt__['grains.append'] = MagicMock()
        ret = grains.list_present(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Failed append value baz to grain foo')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar']})

    # 'list_absent' function tests: 4

    def test_list_absent(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.list_absent(
            name='foo',
            value='bar')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value bar was deleted from grain foo')
        self.assertEqual(ret['changes'], {'deleted': 'bar'})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': []})

    def test_list_absent_inexistent(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval'}
        ret = grains.list_absent(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Grain foo does not exist')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval'})

    def test_list_absent_not_a_list(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': 'bar'}
        ret = grains.list_absent(
            name='foo',
            value='bar')
        # Note from dr4Ke: should be false, IMO
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['comment'], 'Grain foo is not a valid list')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': 'bar'})

    def test_list_absent_already(self):
        grains.__grains__ = grainsmod.__grains__ = {'a': 'aval', 'foo': ['bar']}
        ret = grains.list_absent(
            name='foo',
            value='baz')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['comment'], 'Value baz is absent from grain foo')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(
            grains.__grains__,
            {'a': 'aval', 'foo': ['bar']})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GrainsTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
This module provides the point of entry to SPM, the Salt Package Manager

.. versionadded:: Beryllium
'''

# Import Python libs
from __future__ import absolute_import, print_function
import os
import yaml
import tarfile
import shutil
import msgpack
import sqlite3
import datetime
import hashlib
import logging
import pwd
import grp

# Import Salt libs
import salt.config
import salt.utils
import salt.utils.http as http
import salt.syspaths as syspaths
from salt.ext.six import string_types
from salt.ext.six.moves import input
from salt.ext.six.moves import zip

# Get logging started
log = logging.getLogger(__name__)


class SPMClient(object):
    '''
    Provide an SPM Client
    '''
    def __init__(self, opts=None):  # pylint: disable=W0231
        if not opts:
            opts = salt.config.spm_config(
                os.path.join(syspaths.CONFIG_DIR, 'spm')
            )
        self.opts = opts

    def run(self, args):
        '''
        Run the SPM command
        '''
        command = args[0]
        if command == 'install':
            self._install(args)
        elif command == 'local':
            self._local(args)
        elif command == 'remove':
            self._remove(args)
        elif command == 'build':
            self._build(args)
        elif command == 'update_repo':
            self._download_repo_metadata()
        elif command == 'create_repo':
            self._create_repo(args)
        elif command == 'files':
            self._list_files(args)
        elif command == 'info':
            self._info(args)

    def _local(self, args):
        '''
        Process local commands
        '''
        args.pop(0)
        command = args[0]
        if command == 'install':
            self._local_install(args)
        elif command == 'files':
            self._local_list_files(args)
        elif command == 'info':
            self._local_info(args)

    def _local_install(self, args, pkg_name=None):
        '''
        Install a package from a file
        '''
        if len(args) < 2:
            log.error('A package file must be specified')
            return False

        pkg_file = args[1]

        self._init_db()
        roots_path = self.opts['file_roots']['base'][0]
        pillar_path = self.opts['pillar_roots']['base'][0]
        comps = pkg_file.split('-')
        comps = '-'.join(comps[:-2]).split('/')
        name = comps[-1]

        if not os.path.exists(pkg_file):
            log.error('File {0} not found'.format(pkg_file))
            return False

        if not os.path.exists(roots_path):
            os.makedirs(roots_path)

        sqlite3.enable_callback_tracebacks(True)
        conn = sqlite3.connect(self.opts['spm_db'], isolation_level=None)
        cur = conn.cursor()

        formula_tar = tarfile.open(pkg_file, 'r:bz2')
        formula_ref = formula_tar.extractfile('{0}/FORMULA'.format(name))
        formula_def = yaml.safe_load(formula_ref)

        data = conn.execute('SELECT package FROM packages WHERE package=?', (formula_def['name'], ))
        if data.fetchone() and not self.opts['force']:
            print('Package {0} already installed, not installing again'.format(formula_def['name']))
            return

        if 'dependencies' in formula_def:
            if not isinstance(formula_def['dependencies'], list):
                formula_def['dependencies'] = [formula_def['dependencies']]
            needs = []
            for dep in formula_def['dependencies']:
                if not isinstance(dep, string_types):
                    continue
                data = conn.execute('SELECT package FROM packages WHERE package=?', (dep, ))
                if data.fetchone():
                    continue
                needs.append(dep)
            print('Cannot install {0}, the following dependencies are needed: '
                  '\n\n{1}'.format(formula_def['name'], '\n'.join(needs)))
            return

        if pkg_name is None:
            print('Installing package from file {0}'.format(pkg_file))
        else:
            print('Installing package {0}'.format(pkg_name))

        if not self.opts['assume_yes']:
            res = input('Proceed? [N/y] ')
            if not res.lower().startswith('y'):
                print('... canceled')
                return False

        print('... installing')

        log.debug('Locally installing package file {0} to {1}'.format(pkg_file, roots_path))

        for field in ('version', 'release', 'summary', 'description'):
            if field not in formula_def:
                log.error('Invalid package: the {0} was not found'.format(field))
                return False

        pkg_files = formula_tar.getmembers()
        # First pass: check for files that already exist
        existing_files = []
        for member in pkg_files:
            if member.isdir():
                continue
            if member.name.startswith('{0}/_'.format(name)):
                # Module files are distributed via _modules, _states, etc
                new_name = member.name.replace('{0}/'.format(name), '')
                out_file = os.path.join(roots_path, new_name)
            elif member.name == '{0}/pillar.example'.format(name):
                # Pillars are automatically put in the pillar_roots
                new_name = '{0}.sls.orig'.format(name)
                out_file = os.path.join(pillar_path, new_name)
            else:
                out_file = os.path.join(roots_path, member.name)
            if os.path.exists(out_file):
                existing_files.append(out_file)
                if not self.opts['force']:
                    log.error('{0} already exists, not installing'.format(out_file))

        if existing_files and not self.opts['force']:
            return

        # We've decided to install
        conn.execute('INSERT INTO packages VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', (
            name,
            formula_def['version'],
            formula_def['release'],
            datetime.datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT'),
            formula_def.get('os', None),
            formula_def.get('os_family', None),
            formula_def.get('dependencies', None),
            formula_def.get('os_dependencies', None),
            formula_def.get('os_family_dependencies', None),
            formula_def['summary'],
            formula_def['description'],
        ))

        # No defaults for this in config.py; default to the current running
        # user and group
        uid = self.opts.get('spm_uid', os.getuid())
        gid = self.opts.get('spm_gid', os.getgid())
        uname = pwd.getpwuid(uid)[0]
        gname = grp.getgrgid(gid)[0]

        # Second pass: install the files
        for member in pkg_files:
            out_path = roots_path
            file_ref = formula_tar.extractfile(member)
            member.uid = uid
            member.gid = gid
            member.uname = uname
            member.gname = gname
            if member.isdir():
                digest = ''
            else:
                file_hash = hashlib.sha1()
                file_hash.update(file_ref.read())
                digest = file_hash.hexdigest()
            if member.name.startswith('{0}/_'.format(name)):
                # Module files are distributed via _modules, _states, etc
                member.name = member.name.replace('{0}/'.format(name), '')
            elif member.name == '{0}/pillar.example'.format(name):
                # Pillars are automatically put in the pillar_roots
                member.name = '{0}.sls.orig'.format(name)
                out_path = pillar_path
            formula_tar.extract(member, out_path)
            conn.execute('INSERT INTO files VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', (
                name,
                '{0}/{1}'.format(out_path, member.path),
                member.size,
                member.mode,
                digest,
                member.devmajor,
                member.devminor,
                member.linkname,
                member.linkpath,
                member.uname,
                member.gname,
                member.mtime
            ))
        formula_tar.close()
        conn.close()

    def _traverse_repos(self, callback):
        '''
        Traverse through all repo files and apply the functionality provided in
        the callback to them
        '''
        repo_files = []
        if os.path.exists(self.opts['spm_repos_config']):
            repo_files.append(self.opts['spm_repos_config'])

        for (dirpath, dirnames, filenames) in os.walk('{0}.d'.format(self.opts['spm_repos_config'])):
            for repo_file in filenames:
                if not repo_file.endswith('.repo'):
                    continue
                repo_files.append(repo_file)

        if not os.path.exists(self.opts['spm_cache_dir']):
            os.makedirs(self.opts['spm_cache_dir'])

        for repo_file in repo_files:
            repo_path = '{0}.d/{1}'.format(self.opts['spm_repos_config'], repo_file)
            with salt.utils.fopen(repo_path) as rph:
                repo_data = yaml.safe_load(rph)
                for repo in repo_data:
                    if repo_data[repo].get('enabled', True) is False:
                        continue
                    callback(repo, repo_data[repo])

    def _download_repo_metadata(self):
        '''
        Connect to all repos and download metadata
        '''
        def _update_metadata(repo, repo_info):
            dl_path = '{0}/SPM-METADATA'.format(repo_info['url'])
            if dl_path.startswith('file://'):
                dl_path = dl_path.replace('file://', '')
                with salt.utils.fopen(dl_path, 'r') as rpm:
                    metadata = yaml.safe_load(rpm)
            else:
                response = http.query(
                    '{0}/SPM-METADATA'.format(dl_path),
                )
                metadata = response.get('dict', {})
            cache_path = '{0}/{1}.p'.format(
                self.opts['spm_cache_dir'],
                repo
            )

            with salt.utils.fopen(cache_path, 'w') as cph:
                msgpack.dump(metadata, cph)

        self._traverse_repos(_update_metadata)

    def _get_repo_metadata(self):
        '''
        Return cached repo metadata
        '''
        metadata = {}

        def _read_metadata(repo, repo_info):
            cache_path = '{0}/{1}.p'.format(
                self.opts['spm_cache_dir'],
                repo
            )

            with salt.utils.fopen(cache_path, 'r') as cph:
                metadata[repo] = {
                    'info': repo_info,
                    'packages': msgpack.load(cph),
                }

        self._traverse_repos(_read_metadata)
        return metadata

    def _create_repo(self, args):
        '''
        Scan a directory and create an SPM-METADATA file which describes
        all of the SPM files in that directory.
        '''
        if len(args) < 2:
            log.error('A path to a directory must be specified')
            return False

        if args[1] == '.':
            repo_path = os.environ['PWD']
        else:
            repo_path = args[1]

        repo_metadata = {}
        for (dirpath, dirnames, filenames) in os.walk(repo_path):
            for spm_file in filenames:
                if not spm_file.endswith('.spm'):
                    continue
                spm_path = '{0}/{1}'.format(repo_path, spm_file)
                if not tarfile.is_tarfile(spm_path):
                    continue
                comps = spm_file.split('-')
                spm_name = '-'.join(comps[:-2])
                spm_fh = tarfile.open(spm_path, 'r:bz2')
                formula_handle = spm_fh.extractfile('{0}/FORMULA'.format(spm_name))
                formula_conf = yaml.safe_load(formula_handle.read())
                repo_metadata[spm_name] = {
                    'info': formula_conf.copy(),
                }
                repo_metadata[spm_name]['filename'] = spm_file

        metadata_filename = '{0}/SPM-METADATA'.format(repo_path)
        with salt.utils.fopen(metadata_filename, 'w') as mfh:
            yaml.dump(repo_metadata, mfh, indent=4, canonical=False, default_flow_style=False)

        log.debug('Wrote {0}'.format(metadata_filename))

    def _install(self, args):
        '''
        Install a package from a repo
        '''
        if len(args) < 2:
            log.error('A package must be specified')
            return False

        package = args[1]

        log.debug('Installing package {0}'.format(package))
        repo_metadata = self._get_repo_metadata()
        for repo in repo_metadata:
            repo_info = repo_metadata[repo]
            if package in repo_metadata[repo]['packages']:
                cache_path = '{0}/{1}'.format(
                    self.opts['spm_cache_dir'],
                    repo
                )
                dl_path = '{0}/{1}'.format(repo_info['info']['url'], repo_info['packages'][package]['filename'])
                out_file = '{0}/{1}'.format(cache_path, repo_info['packages'][package]['filename'])
                if not os.path.exists(cache_path):
                    os.makedirs(cache_path)

                if dl_path.startswith('file://'):
                    dl_path = dl_path.replace('file://', '')
                    shutil.copyfile(dl_path, out_file)
                else:
                    http.query(dl_path, text_out=out_file)

                self._local_install((None, out_file), package)
                return

    def _remove(self, args):
        '''
        Remove a package
        '''
        if len(args) < 2:
            log.error('A package must be specified')
            return False

        package = args[1]
        print('Removing package {0}'.format(package))

        if not self.opts['assume_yes']:
            res = input('Proceed? [N/y] ')
            if not res.lower().startswith('y'):
                print('... canceled')
                return False

        print('... removing')

        if not os.path.exists(self.opts['spm_db']):
            log.error('No database at {0}, cannot remove {1}'.format(self.opts['spm_db'], package))
            return

        # Look at local repo index
        sqlite3.enable_callback_tracebacks(True)
        conn = sqlite3.connect(self.opts['spm_db'], isolation_level=None)
        cur = conn.cursor()

        data = conn.execute('SELECT * FROM packages WHERE package=?', (package, ))
        if not data.fetchone():
            log.error('Package {0} not installed'.format(package))
            return

        # Find files that have not changed and remove them
        data = conn.execute('SELECT path, sum FROM files WHERE package=?', (package, ))
        dirs = []
        for filerow in data.fetchall():
            if os.path.isdir(filerow[0]):
                dirs.append(filerow[0])
                continue
            with salt.utils.fopen(filerow[0], 'r') as fh_:
                file_hash = hashlib.sha1()
                file_hash.update(fh_.read())
                digest = file_hash.hexdigest()
                if filerow[1] == digest:
                    log.trace('Removing file {0}'.format(filerow[0]))
                    os.remove(filerow[0])
                else:
                    log.trace('Not removing file {0}'.format(filerow[0]))
                conn.execute('DELETE FROM files WHERE path=?', (filerow[0], ))

        # Clean up directories
        for dir_ in sorted(dirs, reverse=True):
            conn.execute('DELETE FROM files WHERE path=?', (dir_, ))
            try:
                log.trace('Removing directory {0}'.format(dir_))
                os.rmdir(dir_)
            except OSError:
                # Leave directories in place that still have files in them
                log.trace('Cannot remove directory {0}, probably not empty'.format(dir_))

        conn.execute('DELETE FROM packages WHERE package=?', (package, ))

    def _local_info(self, args):
        '''
        List info for a package file
        '''
        if len(args) < 2:
            log.error('A package filename must be specified')
            return False

        pkg_file = args[1]

        comps = pkg_file.split('-')
        comps = '-'.join(comps[:-2]).split('/')
        name = comps[-1]

        formula_tar = tarfile.open(pkg_file, 'r:bz2')
        formula_ref = formula_tar.extractfile('{0}/FORMULA'.format(name))
        formula_def = yaml.safe_load(formula_ref)

        self._print_info(formula_def)

    def _info(self, args):
        '''
        List info for a package
        '''
        if len(args) < 2:
            log.error('A package must be specified')
            return False

        package = args[1]

        conn = sqlite3.connect(self.opts['spm_db'], isolation_level=None)
        cur = conn.cursor()

        fields = (
            'package',
            'version',
            'release',
            'installed',
            'os',
            'os_family',
            'dependencies',
            'os_dependencies',
            'os_family_dependencies',
            'summary',
            'description',
        )
        data = conn.execute(
            'SELECT {0} FROM packages WHERE package=?'.format(','.join(fields)),
            (package, )
        )
        row = data.fetchone()
        if not row:
            print('Package {0} not installed'.format(package))
            return

        formula_def = dict(list(zip(fields, row)))
        formula_def['name'] = formula_def['package']

        self._print_info(formula_def)

    def _print_info(self, formula_def):
        '''
        Print package info
        '''
        fields = (
            'name',
            'os',
            'os_family',
            'release',
            'version',
            'dependencies',
            'os_dependencies',
            'os_family_dependencies',
            'summary',
            'description',
        )
        for item in fields:
            if item not in formula_def:
                formula_def[item] = 'None'

        if 'installed' not in formula_def:
            formula_def['installed'] = 'Not installed'

        print('''Name: {name}
Version: {version}
Release: {release}
Install Date: {installed}
Supported OSes: {os}
Supported OS families: {os_family}
Dependencies: {dependencies}
OS Dependencies: {os_dependencies}
OS Family Dependencies: {os_family_dependencies}
Summary: {summary}
Description:
{description}
'''.format(**formula_def))

    def _local_list_files(self, args):
        '''
        List files for a package file
        '''
        if len(args) < 2:
            log.error('A package filename must be specified')
            return False

        pkg_file = args[1]
        formula_tar = tarfile.open(pkg_file, 'r:bz2')
        pkg_files = formula_tar.getmembers()

        for member in pkg_files:
            print(member.name)

    def _list_files(self, args):
        '''
        List files for an installed package
        '''
        if len(args) < 2:
            log.error('A package name must be specified')
            return False

        package = args[1]

        conn = sqlite3.connect(self.opts['spm_db'], isolation_level=None)
        cur = conn.cursor()

        data = conn.execute('SELECT package FROM packages WHERE package=?', (package, ))
        if not data.fetchone():
            print('Package {0} not installed'.format(package))
            return

        data = conn.execute('SELECT path FROM files WHERE package=?', (package, ))
        for file_ in data.fetchall():
            print(file_[0])

    def _build(self, args):
        '''
        Build a package
        '''
        if len(args) < 2:
            log.error('A path to a formula must be specified')
            return False

        self.abspath = args[1].rstrip('/')
        comps = self.abspath.split('/')
        self.relpath = comps[-1]

        formula_path = '{0}/FORMULA'.format(self.abspath)
        formula_conf = {}
        if os.path.exists(formula_path):
            with salt.utils.fopen(formula_path) as fp_:
                formula_conf = yaml.safe_load(fp_)
        else:
            log.debug('File not found')
            return False

        for field in ('version', 'release', 'summary', 'description'):
            if field not in formula_conf:
                log.error('Invalid package: a {0} must be defined'.format(field))
                return False

        out_path = '{0}/{1}-{2}-{3}.spm'.format(
            self.opts['spm_build_dir'],
            formula_conf['name'],
            formula_conf['version'],
            formula_conf['release'],
        )

        if not os.path.exists(self.opts['spm_build_dir']):
            os.mkdir(self.opts['spm_build_dir'])

        formula_tar = tarfile.open(out_path, 'w:bz2')
        formula_tar.add(self.abspath, formula_conf['name'], exclude=self._exclude)
        formula_tar.close()

        log.debug(formula_path)
        return formula_path

    def _exclude(self, name):
        '''
        Exclude based on opts
        '''
        for item in self.opts['spm_build_exclude']:
            exclude_name = '{0}/{1}'.format(self.abspath, item)
            if name.startswith(exclude_name):
                return True
        return False

    def _init_db(self):
        '''
        Initialize the package database
        '''
        if not os.path.exists(self.opts['spm_db']):
            log.debug('Creating new package database at {0}'.format(self.opts['spm_db']))
            conn = sqlite3.connect(self.opts['spm_db'], isolation_level=None)
            cur = conn.cursor()
            conn.execute('''CREATE TABLE packages (
                package text,
                version text,
                release text,
                installed text,
                os text,
                os_family text,
                dependencies text,
                os_dependencies text,
                os_family_dependencies text,
                summary text,
                description text
            )''')
            conn.execute('''CREATE TABLE files (
                package text,
                path text,
                size real,
                mode text,
                sum text,
                major text,
                minor text,
                linkname text,
                linkpath text,
                uname text,
                gname text,
                mtime text
            )''')
            conn.close()



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import reg


class MockWinReg(object):
    '''
        Mock class of winreg
    '''
    HKEY_USERS = "HKEY_USERS"
    HKEY_CURRENT_USER = "HKEY_CURRENT_USER"
    HKEY_LOCAL_MACHINE = "HKEY_LOCAL_MACHINE"
    KEY_ALL_ACCESS = True
    KEY_WOW64_64KEY = False
    flag = None
    flag1 = None
    flag2 = None

    def __init__(self):
        pass

    def OpenKeyEx(self, hkey2, path, bol, access_mask):
        '''
            Mock openKeyEx method
        '''
        if self.flag:
            return hkey2, path, bol, access_mask
        else:
            raise Exception("Error")

    @staticmethod
    def QueryValueEx(handle, key):
        '''
            Mock QueryValueEx method
        '''
        return [handle, key]

    def OpenKey(self, hkey2, path, bol, access_mask):
        '''
            Mock OpenKey Mothod
        '''
        if self.flag:
            return hkey2, path, bol, access_mask
        else:
            raise Exception("Error")

    @staticmethod
    def SetValueEx(handle, key, bol, _type, value):
        '''
            Mock SetValueEx method
        '''
        return handle, key, bol, _type, value

    @staticmethod
    def CloseKey(handle):
        '''
            Mock CloseKey method
        '''
        return handle

    @staticmethod
    def CreateKeyEx(hkey2, path, bol, access_mask):
        '''
            Mock CreateKeyEx method
        '''
        return hkey2, path, bol, access_mask

    def DeleteKeyEx(self, handle, key):
        '''
            Mock DeleteKeyEx method
        '''
        if self.flag1:
            return handle, key
        else:
            raise Exception("Error")

    def DeleteValue(self, handle, key):
        '''
            Mock DeleteValue
        '''
        if self.flag2:
            return handle, key
        else:
            raise Exception("Error")

reg._winreg = MockWinReg()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RegTestCase(TestCase):
    '''
        Test cases for salt.modules.reg
    '''
    def test_read_key(self):
        '''
            Test to read registry key value
        '''
        MockWinReg.flag = False
        self.assertEqual(reg.read_key("HKEY_LOCAL_MACHINE",
                                      "SOFTWARE\\Salt",
                                      "2014.7.0"),
                         None)

        MockWinReg.flag = True
        self.assertTrue(reg.read_key("HKEY_LOCAL_MACHINE",
                                     "SOFTWARE\\Salt",
                                     "2014.7.0"))

    def test_set_key(self):
        '''
            Test to set a registry key
        '''
        self.assertFalse(reg.set_key("HKEY_CURRENT_USER",
                                     "SOFTWARE\\Salt",
                                     "2014.7.0",
                                     "0.97")
                         )

        MockWinReg.flag = True
        self.assertTrue(reg.set_key("HKEY_CURRENT_USER",
                                    "SOFTWARE\\Salt",
                                    "2014.7.0",
                                    "0.97",
                                    "OpenKey")
                        )

        MockWinReg.flag = False
        self.assertTrue(reg.set_key("HKEY_CURRENT_USER",
                                    "SOFTWARE\\Salt",
                                    "2014.7.0",
                                    "0.97",
                                    "OpenKey")
                        )

    def test_create_key(self):
        '''
            Test to Create a registry key
        '''
        MockWinReg.flag = True
        self.assertTrue(reg.create_key("HKEY_CURRENT_USER",
                                       "SOFTWARE\\Salt",
                                       "2014.7.0"
                                       )
                        )

        MockWinReg.flag = False
        self.assertTrue(reg.create_key("HKEY_CURRENT_USER",
                                       "SOFTWARE\\Salt",
                                       "2014.7.0"
                                       )
                        )

    def test_delete_key(self):
        '''
            Test to delete key
        '''
        MockWinReg.flag = True
        MockWinReg.flag1 = True
        self.assertTrue(reg.delete_key("HKEY_CURRENT_USER",
                                       "SOFTWARE\\Salt",
                                       "2014.7.0"
                                       )
                        )

        MockWinReg.flag = True
        MockWinReg.flag1 = False
        MockWinReg.flag2 = False
        self.assertFalse(reg.delete_key("HKEY_CURRENT_USER",
                                        "SOFTWARE\\Salt",
                                        "2014.7.0"
                                        )
                         )

        MockWinReg.flag = True
        MockWinReg.flag1 = False
        MockWinReg.flag2 = True
        self.assertTrue(reg.delete_key("HKEY_CURRENT_USER",
                                       "SOFTWARE\\Salt",
                                       "2014.7.0"
                                       )
                        )

if __name__ == '__main__':
    from integration import run_tests
    run_tests(RegTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
A convenience system to manage jobs, both active and already run
'''

# Import python libs
from __future__ import absolute_import, print_function
import fnmatch
import logging
import os

# Import salt libs
import salt.client
import salt.payload
import salt.utils
import salt.utils.jid
import salt.minion

# Import 3rd-party libs
import salt.ext.six as six
from salt.exceptions import SaltClientError

try:
    import dateutil.parser as dateutil_parser
    DATEUTIL_SUPPORT = True
except ImportError:
    DATEUTIL_SUPPORT = False

log = logging.getLogger(__name__)


def active(outputter=None, display_progress=False):
    '''
    Return a report on all actively running jobs from a job id centric
    perspective

    CLI Example:

    .. code-block:: bash

        salt-run jobs.active
    '''
    ret = {}
    client = salt.client.get_local_client(__opts__['conf_file'])
    try:
        active_ = client.cmd('*', 'saltutil.running', timeout=__opts__['timeout'])
    except SaltClientError as client_error:
        print(client_error)
        return ret

    if display_progress:
        __jid_event__.fire_event({
            'message': 'Attempting to contact minions: {0}'.format(list(active_.keys()))
            }, 'progress')
    for minion, data in six.iteritems(active_):
        if display_progress:
            __jid_event__.fire_event({'message': 'Received reply from minion {0}'.format(minion)}, 'progress')
        if not isinstance(data, list):
            continue
        for job in data:
            if not job['jid'] in ret:
                ret[job['jid']] = _format_jid_instance(job['jid'], job)
                ret[job['jid']].update({'Running': [{minion: job.get('pid', None)}], 'Returned': []})
            else:
                ret[job['jid']]['Running'].append({minion: job['pid']})

    mminion = salt.minion.MasterMinion(__opts__)
    for jid in ret:
        returner = _get_returner((__opts__['ext_job_cache'], __opts__['master_job_cache']))
        data = mminion.returners['{0}.get_jid'.format(returner)](jid)
        for minion in data:
            if minion not in ret[jid]['Returned']:
                ret[jid]['Returned'].append(minion)

    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the jobs.active runner '
            'has been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': ret}
    else:
        return ret


def lookup_jid(jid,
               ext_source=None,
               returned=True,
               missing=False,
               outputter=None,
               display_progress=False):
    '''
    Return the printout from a previously executed job

    jid
        The jid to look up.

    ext_source
        The external job cache to use. Default: `None`.

    returned
        When set to `True`, adds the minions that did return from the command.
        Default: `True`.

        .. versionadded:: Beryllium

    missing
        When set to `True`, adds the minions that did NOT return from the command.
        Default: `False`.

    display_progress
        Displays progress events when set to `True`. Default: `False`.

        .. versionadded:: 2015.5.0

    CLI Example:

    .. code-block:: bash

        salt-run jobs.lookup_jid 20130916125524463507
        salt-run jobs.lookup_jid 20130916125524463507 outputter=highstate
    '''
    ret = {}
    mminion = salt.minion.MasterMinion(__opts__)
    returner = _get_returner((__opts__['ext_job_cache'], ext_source, __opts__['master_job_cache']))
    if display_progress:
        __jid_event__.fire_event({'message': 'Querying returner: {0}'.format(returner)}, 'progress')

    try:
        data = mminion.returners['{0}.get_jid'.format(returner)](jid)
    except TypeError:
        return 'Requested returner could not be loaded. No JIDs could be retrieved.'

    for minion in data:
        if display_progress:
            __jid_event__.fire_event({'message': minion}, 'progress')
        if u'return' in data[minion]:
            if returned:
                ret[minion] = data[minion].get(u'return')
        else:
            if returned:
                ret[minion] = data[minion].get('return')
    if missing:
        load = mminion.returners['{0}.get_load'.format(returner)](jid)
        ckminions = salt.utils.minions.CkMinions(__opts__)
        exp = ckminions.check_minions(load['tgt'], load['tgt_type'])
        for minion_id in exp:
            if minion_id not in data:
                ret[minion_id] = 'Minion did not return'

    # Once we remove the outputter argument in a couple releases, we still
    # need to check to see if the 'out' key is present and use it to specify
    # the correct outputter, so we get highstate output for highstate runs.
    if outputter is None:
        try:
            # Check if the return data has an 'out' key. We'll use that as the
            # outputter in the absence of one being passed on the CLI.
            outputter = data[next(iter(data))].get('out')
        except (StopIteration, AttributeError):
            outputter = None
    else:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the jobs.lookup_jid runner '
            'has been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )

    if outputter:
        return {'outputter': outputter, 'data': ret}
    else:
        return ret


def list_job(jid, ext_source=None, outputter=None):
    '''
    List a specific job given by its jid

    CLI Example:

    .. code-block:: bash

        salt-run jobs.list_job 20130916125524463507
    '''
    ret = {'jid': jid}
    mminion = salt.minion.MasterMinion(__opts__)
    returner = _get_returner((__opts__['ext_job_cache'], ext_source, __opts__['master_job_cache']))

    job = mminion.returners['{0}.get_load'.format(returner)](jid)
    ret.update(_format_jid_instance(jid, job))
    ret['Result'] = mminion.returners['{0}.get_jid'.format(returner)](jid)
    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the jobs.list_job runner '
            'has been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': ret}
    else:
        return ret


def list_jobs(ext_source=None,
              outputter=None,
              search_metadata=None,
              search_function=None,
              search_target=None,
              start_time=None,
              end_time=None,
              display_progress=False):
    '''
    List all detectable jobs and associated functions

    ext_source
        The external job cache to use. Default: `None`.

    search_metadata
        Search the metadata of a job for the provided string of dictionary.
        Default: 'None'.

    search_function
        Search the function of a job for the provided string.
        Default: 'None'.

    search_target
        Search the target of a job for the provided minion name.
        Default: 'None'.

    start_time
        Search for jobs where the start time of the job is greater than
        or equal to the provided time stamp.  Any timestamp supported
        by the Dateutil (required) module can be used.
        Default: 'None'.

    end_time
        Search for jobs where the start time of the job is less than
        or equal to the provided time stamp.  Any timestamp supported
        by the Dateutil (required) module can be used.
        Default: 'None'.

    CLI Example:

    .. code-block:: bash

        salt-run jobs.list_jobs
        salt-run jobs.list_jobs search_function='test.*' search_target='localhost' search_metadata='{"bar": "foo"}'
        salt-run jobs.list_jobs start_time='2015, Mar 16 19:00' end_time='2015, Mar 18 22:00'

    '''
    returner = _get_returner((__opts__['ext_job_cache'], ext_source, __opts__['master_job_cache']))
    if display_progress:
        __jid_event__.fire_event({'message': 'Querying returner {0} for jobs.'.format(returner)}, 'progress')
    mminion = salt.minion.MasterMinion(__opts__)

    ret = mminion.returners['{0}.get_jids'.format(returner)]()

    mret = {}
    for item in ret:
        _match = True
        if search_metadata:
            _match = False
            if 'Metadata' in ret[item]:
                if isinstance(search_metadata, dict):
                    for key in search_metadata:
                        if key in ret[item]['Metadata']:
                            if ret[item]['Metadata'][key] == search_metadata[key]:
                                _match = True
                else:
                    log.info('The search_metadata parameter must be specified'
                             ' as a dictionary.  Ignoring.')
        if search_target and _match:
            _match = False
            if 'Target' in ret[item]:
                if isinstance(search_target, list):
                    for key in search_target:
                        if fnmatch.fnmatch(ret[item]['Target'], key):
                            _match = True
                elif isinstance(search_target, six.string_types):
                    if fnmatch.fnmatch(ret[item]['Target'], search_target):
                        _match = True

        if search_function and _match:
            _match = False
            if 'Function' in ret[item]:
                if isinstance(search_function, list):
                    for key in search_function:
                        if fnmatch.fnmatch(ret[item]['Function'], key):
                            _match = True
                elif isinstance(search_function, six.string_types):
                    if fnmatch.fnmatch(ret[item]['Function'], search_function):
                        _match = True

        if start_time and _match:
            _match = False
            if DATEUTIL_SUPPORT:
                parsed_start_time = dateutil_parser.parse(start_time)
                _start_time = dateutil_parser.parse(ret[item]['StartTime'])
                if _start_time >= parsed_start_time:
                    _match = True
            else:
                log.error('"dateutil" library not available, skipping start_time comparision.')

        if end_time and _match:
            _match = False
            if DATEUTIL_SUPPORT:
                parsed_end_time = dateutil_parser.parse(end_time)
                _start_time = dateutil_parser.parse(ret[item]['StartTime'])
                if _start_time <= parsed_end_time:
                    _match = True
            else:
                log.error('"dateutil" library not available, skipping start_time comparision.')

        if _match:
            mret[item] = ret[item]

    if outputter:
        return {'outputter': outputter, 'data': mret}
    else:
        return mret


def print_job(jid, ext_source=None, outputter=None):
    '''
    Print a specific job's detail given by it's jid, including the return data.

    CLI Example:

    .. code-block:: bash

        salt-run jobs.print_job 20130916125524463507
    '''
    ret = {}

    returner = _get_returner((__opts__['ext_job_cache'], ext_source, __opts__['master_job_cache']))
    mminion = salt.minion.MasterMinion(__opts__)

    try:
        job = mminion.returners['{0}.get_load'.format(returner)](jid)
        ret[jid] = _format_jid_instance(jid, job)
    except TypeError:
        ret[jid]['Result'] = ('Requested returner {0} is not available. Jobs cannot be retrieved. '
            'Check master log for details.'.format(returner))
        return ret
    ret[jid]['Result'] = mminion.returners['{0}.get_jid'.format(returner)](jid)
    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the jobs.print_job runner '
            'has been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': ret}
    else:
        return ret


def last_run(ext_source=None,
             outputter=None,
             metadata=None,
             function=None,
             target=None,
             display_progress=False):
    '''
    List all detectable jobs and associated functions

    .. versionadded:: Beryllium

    CLI Example:

    .. code-block:: bash

        salt-run jobs.last_run

        salt-run jobs.last_run target=nodename

        salt-run jobs.last_run function='cmd.run'

        salt-run jobs.last_run metadata="{'foo': 'bar'}"
    '''

    if metadata:
        if not isinstance(metadata, dict):
            log.info('The metadata parameter must be specified as a dictionary')
            return False

    _all_jobs = list_jobs(ext_source, outputter, metadata, function, target, display_progress)
    if _all_jobs:
        last_job = sorted(_all_jobs)[-1]
        return print_job(last_job, ext_source, outputter)
    else:
        return False


def _get_returner(returner_types):
    '''
    Helper to iterate over returner_types and pick the first one
    '''
    for returner in returner_types:
        if returner and returner is not None:
            return returner


def _format_job_instance(job):
    '''
    Helper to format a job instance
    '''
    ret = {'Function': job.get('fun', 'unknown-function'),
           'Arguments': list(job.get('arg', [])),
           # unlikely but safeguard from invalid returns
           'Target': job.get('tgt', 'unknown-target'),
           'Target-type': job.get('tgt_type', []),
           'User': job.get('user', 'root')}

    if 'metadata' in job:
        ret['Metadata'] = job.get('metadata', {})
    else:
        if 'kwargs' in job:
            if 'metadata' in job['kwargs']:
                ret['Metadata'] = job['kwargs'].get('metadata', {})

    if 'Minions' in job:
        ret['Minions'] = job['Minions']
    return ret


def _format_jid_instance(jid, job):
    '''
    Helper to format jid instance
    '''
    ret = _format_job_instance(job)
    ret.update({'StartTime': salt.utils.jid.jid_to_time(jid)})
    return ret


def _walk_through(job_dir, display_progress=False):
    '''
    Walk through the job dir and return jobs
    '''
    serial = salt.payload.Serial(__opts__)

    for top in os.listdir(job_dir):
        t_path = os.path.join(job_dir, top)

        for final in os.listdir(t_path):
            load_path = os.path.join(t_path, final, '.load.p')
            job = serial.load(salt.utils.fopen(load_path, 'rb'))

            if not os.path.isfile(load_path):
                continue

            job = serial.load(salt.utils.fopen(load_path, 'rb'))
            jid = job['jid']
            if display_progress:
                __jid_event__.fire_event({'message': 'Found JID {0}'.format(jid)}, 'progress')
            yield jid, job, t_path, final



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import scsi
import os


# Globals
scsi.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ScsiTestCase(TestCase):
    '''
    Test cases for salt.modules.scsi
    '''
    def test_ls_(self):
        '''
        Test for list SCSI devices, with details
        '''
        with patch.dict(scsi.__salt__,
                        {'cmd.run':
                         MagicMock(return_value='[A:a B:b C:c D:d]')}):
            self.assertDictEqual(scsi.ls_(),
                                 {'[A:a':
                                  {'major': 'C', 'lun': 'A:a', 'device': 'B:b',
                                   'model': '', 'minor': 'c', 'size': 'D:d]'}})

    def test_rescan_all(self):
        '''
        Test for list scsi devices
        '''
        mock = MagicMock(side_effect=[False, True])
        with patch.object(os.path, 'isdir', mock):
            self.assertEqual(scsi.rescan_all('host'),
                             'Host host does not exist')

            with patch.dict(scsi.__salt__,
                            {'cmd.run': MagicMock(return_value='A')}):
                self.assertListEqual(scsi.rescan_all('host'), ['A'])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ScsiTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import glusterfs
import salt.utils.cloud as suc

# Globals
glusterfs.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GitTestCase(TestCase):
    '''
    Test cases for salt.modules.glusterfs
    '''
    # 'list_peers' function tests: 1

    def test_list_peers(self):
        '''
        Test if it return a list of gluster peers
        '''
        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertListEqual(glusterfs.list_peers(), [])

        mock = MagicMock(return_value='No peers present')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertIsNone(glusterfs.list_peers())

    # 'peer' function tests: 1

    def test_peer(self):
        '''
        Test if it add another node into the peer list.
        '''
        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.peer('salt'), '')

        mock = MagicMock(return_value=True)
        with patch.object(suc, 'check_name', mock):
            self.assertEqual(glusterfs.peer('a'),
                             'Invalid characters in peer name')

    # 'create' function tests: 1

    def test_create(self):
        '''
        Test if it create a glusterfs volume.
        '''
        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.create('newvolume', 'host1:brick'),
                             'Error: Brick paths must start with /')

        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.create('newvolume', 'host1/brick'),
                             'Error: Brick syntax is <peer>:<path>')

        mock = MagicMock(return_value='creation success')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.create('newvolume', 'host1:/brick',
                                              True, True, True, 'tcp', True),
                             'Volume newvolume created and started')

        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.create('newvolume', 'host1:/brick',
                                              True, True, True,
                                              'tcp', True), '')

        mock = MagicMock(return_value='')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.create('newvolume', 'host1:/brick'),
                             'Volume newvolume created. Start volume to use')

    # 'list_volumes' function tests: 1

    def test_list_volumes(self):
        '''
        Test if it list configured volumes
        '''
        mock = MagicMock(return_value='No volumes present in cluster')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertListEqual(glusterfs.list_volumes(), [])

        mock = MagicMock(return_value='Newvolume1\nNewvolume2')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertListEqual(glusterfs.list_volumes(),
                                 ['Newvolume1', 'Newvolume2'])

    # 'status' function tests: 1

    def test_status(self):
        '''
        Test if it check the status of a gluster volume.
        '''
        mock = MagicMock(return_value='No volumes present in cluster')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertDictEqual(glusterfs.status('myvolume'),
                                 {'bricks': {}, 'healers': {}, 'nfs': {}})

        mock = MagicMock(return_value='does not exist\n')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.status('myvolume'), 'does not exist')

        mock = MagicMock(return_value='is not started')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.status('myvolume'), 'is not started')

    # 'start_volume' function tests: 1

    def test_start_volume(self):
        '''
        Test if it start a gluster volume.
        '''
        mock = MagicMock(return_value=['Newvolume1', 'Newvolume2'])
        with patch.object(glusterfs, 'list_volumes', mock):
            mock = MagicMock(return_value='creation success')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.start_volume('Newvolume1'),
                                 'Volume already started')

            mock = MagicMock(side_effect=['does not exist',
                                          'creation success'])
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.start_volume('Newvolume1'),
                                 'Volume Newvolume1 started')

            mock = MagicMock(return_value='does not exist')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.start_volume('Newvolume1'),
                                 'does not exist')

        mock = MagicMock(return_value='No volumes present in cluster')
        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
            self.assertEqual(glusterfs.start_volume('mycluster'),
                             'Volume does not exist')

    # 'stop_volume' function tests: 1

    def test_stop_volume(self):
        '''
        Test if it stop a gluster volume.
        '''
        mock = MagicMock(return_value={})
        with patch.object(glusterfs, 'status', mock):
            mock = MagicMock(return_value='creation success')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.stop_volume('Newvolume1'),
                                 'Volume Newvolume1 stopped')

            mock = MagicMock(return_value='No volume exist')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.stop_volume('Newvolume1'),
                                 'No volume exist')

        mock = MagicMock(return_value='')
        with patch.object(glusterfs, 'status', mock):
            self.assertEqual(glusterfs.stop_volume('Newvolume1'), '')

    # 'delete' function tests: 1

    def test_delete(self):
        '''
        Test if it deletes a gluster volume.
        '''
        ret = 'Error: Volume must be stopped before deletion'
        mock = MagicMock(return_value=['Newvolume1', 'Newvolume2'])
        with patch.object(glusterfs, 'list_volumes', mock):
            self.assertEqual(glusterfs.delete('Newvolume3'),
                             'Volume does not exist')

            mock = MagicMock(return_value='creation success')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.delete('Newvolume1', False), ret)

            mock = MagicMock(return_value='creation success')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.delete('Newvolume1'),
                                 'Volume Newvolume1 stopped and deleted')

            mock = MagicMock(return_value='')
            with patch.object(glusterfs, 'status', mock):
                mock = MagicMock(return_value='creation success')
                with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                    self.assertEqual(glusterfs.delete('Newvolume1'),
                                     'Volume Newvolume1 deleted')

            mock = MagicMock(return_value='does not exist')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.delete('Newvolume1'),
                                 'does not exist')

    # 'add_volume_bricks' function tests: 1

    def test_add_volume_bricks(self):
        '''
        Test if it add brick(s) to an existing volume
        '''
        mock = MagicMock(return_value='does not exist')
        with patch.object(glusterfs, 'status', mock):
            self.assertEqual(glusterfs.add_volume_bricks('Newvolume1',
                                                         ['bricks']),
                             'does not exist')

        mock = MagicMock(return_value='is not started')
        with patch.object(glusterfs, 'status', mock):
            self.assertEqual(glusterfs.add_volume_bricks('Newvolume1',
                                                         ['bricks']),
                             'is not started')

        ret = '1 bricks successfully added to the volume Newvolume1'
        mock = MagicMock(return_value={'bricks': {}, 'healers': {}, 'nfs': {}})
        with patch.object(glusterfs, 'status', mock):
            mock = MagicMock(return_value='creation success')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.add_volume_bricks('Newvolume1',
                                                             ['bricks']), ret)

            mock = MagicMock(return_value='')
            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
                self.assertEqual(glusterfs.add_volume_bricks('Newvolume1',
                                                             ['bricks']), '')

                self.assertEqual(glusterfs.add_volume_bricks('Newvolume1', []),
                                 'Bricks already in volume Newvolume1')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GitTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath
from mock import mock_open

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import gpg
from salt.exceptions import SaltInvocationError

gpg.__salt__ = {}

RET = [{'created': '2014-07-25',
        'fingerprint': u'F321F',
        'keyLength': u'1024',
        'keyid': u'3F0C8E90D459D89A',
        'ownerTrust': 'Ultimately Trusted',
        'trust': 'u',
        'uids': [u'Autogenerated Key (Generated by SaltStack)']}]


class Mockgnupg(object):
    '''
    Mock gnupg class
    '''
    __version__ = '1.3.1'
    fingerprint = u'F321F'
    counts = {}
    count = ''
    imported = False
    imported_rsa = False
    results = [{'ok': '1', 'fingerprint': u'F321F'}]
    data = True
    trust_level = None
    ok = True
    unchanged = False
    not_imported = False

    class GPG(object):
        '''
        Mock gnupg class
        '''
        def __init__(self, gnupghome='/tmp/salt/.gnupg',
                     homedir='/tmp/salt/.gnupg'):
            self.gnupghome = gnupghome
            self.homedir = homedir
            self.text = None
            self.keyserver = None
            self.kwargs = None
            self.obj = None
            self.fingerprints = None
            self.keyserver = None
            self.secret = None
            self.keyids = None
            self.default_key = None
            self.recipients = None
            self.passphrase = None

        def search_keys(self, text, keyserver):
            '''
            Mock of search_keys method
            '''
            self.text = text
            self.keyserver = keyserver
            return RET

        def gen_key_input(self, **kwargs):
            '''
            Mock of gen_key_input method
            '''
            self.kwargs = kwargs
            return Mockgnupg

        def gen_key(self, obj):
            '''
            Mock of gen_key method
            '''
            self.obj = obj
            return Mockgnupg

        def list_keys(self, obj):
            '''
            Mock of list_keys method
            '''
            self.obj = obj
            return RET

        def delete_keys(self, fingerprints, secret=False):
            '''
            Mock of delete_keys method
            '''
            self.fingerprints = fingerprints
            self.secret = secret
            return 'ok'

        def import_keys(self, text):
            '''
            Mock of import_keys method
            '''
            self.text = text
            return Mockgnupg

        def export_keys(self, keyids, secret):
            '''
            Mock of export_keys method
            '''
            self.secret = secret
            self.keyids = keyids
            return (keyids, secret)

        def recv_keys(self, keyserver, *keyids):
            '''
            Mock of recv_keys method
            '''
            self.keyserver = keyserver
            self.keyids = keyids
            return Mockgnupg

        def sign(self, text, default_key, passphrase):
            '''
            Mock of sign method
            '''
            self.text = text
            self.default_key = default_key
            self.passphrase = passphrase
            return Mockgnupg

        def verify(self, text):
            '''
            Mock of verify method
            '''
            self.text = text
            return Mockgnupg

        def encrypt(self, text, recipients, passphrase):
            '''
            Mock of encrypt method
            '''
            self.text = text
            self.recipients = recipients
            self.passphrase = passphrase
            return Mockgnupg

        def decrypt(self, text, passphrase):
            '''
            Mock of decrypt method
            '''
            self.text = text
            self.passphrase = passphrase
            return Mockgnupg

gpg.gnupg = Mockgnupg()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GpgTestCase(TestCase):
    '''
    TestCase for salt.modules.gpg
    '''
    # 'search_keys' function tests: 1

    def test_search_keys(self):
        '''
        Tests if it search keys from keyserver.
        '''
        ret = [{'keyid': u'3F0C8E90D459D89A',
                'uids': [u'Autogenerated Key (Generated by SaltStack)']}]
        mock = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'user.info': mock}):
            self.assertListEqual(gpg.search_keys('user@example.com',
                                                 user='username'), ret)

            gpg.GPG_1_3_1 = True
            self.assertRaises(SaltInvocationError, gpg.search_keys,
                              'user@example.com')

    # 'list_keys' function tests: 1

    def test_list_keys(self):
        '''
        Tests if it list keys in GPG keychain
        '''
        ret = [{'fingerprint': u'F321F', 'keyid': u'3F0C8E90D459D89A',
                'trust': 'Ultimately Trusted',
                'uids': [u'Autogenerated Key (Generated by SaltStack)']}]

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertListEqual(gpg.list_keys(), ret)

    # 'list_secret_keys' function tests: 1

    def test_list_secret_keys(self):
        '''
        Tests if it list secret keys in GPG keychain
        '''
        ret = [{'fingerprint': u'F321F', 'keyid': u'3F0C8E90D459D89A',
                'trust': 'Ultimately Trusted',
                'uids': [u'Autogenerated Key (Generated by SaltStack)']}]

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertListEqual(gpg.list_secret_keys(), ret)

    # 'create_key' function tests: 1

    def test_create_key(self):
        '''
        Tests if it create a key in the GPG keychain
        '''
        ret = {'res': True, 'fingerprint': u'F321F',
               'message': 'GPG key pair successfully generated.'}

        ret1 = {'fingerprint': '', 'res': False,
                'message': 'gpg_passphrase not available in pillar.'}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_item = MagicMock(return_value=False)
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'pillar.item': mock_item}):
            self.assertDictEqual(gpg.create_key(), ret)

            self.assertDictEqual(gpg.create_key(use_passphrase=True), ret1)

    # 'delete_key' function tests: 1

    def test_delete_key(self):
        '''
        Tests if it delete a key from the GPG keychain
        '''
        ret = {'message': 'Only specify one argument, fingerprint or keyid',
               'res': False}

        ret1 = {'message': 'Required argument, fingerprint or keyid',
                'res': False}

        ret2 = {'message': ('Secret key exists, delete first'
                            ' or pass delete_secret=True.'), 'res': False}

        ret3 = {'message': ('Secret key for F321F deleted\nPublic'
                            ' key for F321F deleted'), 'res': True}

        ret4 = {'message': 'Key not available in keychain.', 'res': False}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertDictEqual(gpg.delete_key(keyid='3FAD9F1E',
                                                fingerprint='53C'), ret)

            self.assertDictEqual(gpg.delete_key(), ret1)

            self.assertDictEqual(gpg.delete_key(keyid='3F0C8E90D459D89A'), ret2)

            self.assertDictEqual(gpg.delete_key(keyid='3F0C8E90D459D89A',
                                                delete_secret=True), ret3)

            self.assertDictEqual(gpg.delete_key(keyid='3F0C'), ret4)

    # 'get_key' function tests: 1

    def test_get_key(self):
        '''
        Tests if it get a key from the GPG keychain
        '''
        ret = {'fingerprint': u'F321F', 'keyid': u'3F0C8E90D459D89A',
               'trust': 'Ultimately Trusted',
               'uids': [u'Autogenerated Key (Generated by SaltStack)']}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertFalse(gpg.get_key())

            self.assertDictEqual(gpg.get_key(keyid='3F0C8E90D459D89A'), ret)

    # 'get_secret_key' function tests: 1

    def test_get_secret_key(self):
        '''
        Tests if it get a secret key from the GPG keychain
        '''
        ret = {'fingerprint': u'F321F', 'keyid': u'3F0C8E90D459D89A',
               'trust': 'Ultimately Trusted',
               'uids': [u'Autogenerated Key (Generated by SaltStack)']}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertFalse(gpg.get_secret_key())

            self.assertDictEqual(gpg.get_secret_key(keyid='3F0C8E90D459D89A'),
                                 ret)

    # 'import_key' function tests: 1

    def test_import_key(self):
        '''
        Tests if it import a key from text or file.
        '''
        ret = {'message': 'Unable to import key.', 'res': False}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertRaises(SaltInvocationError, gpg.import_key)

            with patch('salt.utils.flopen', mock_open(read_data='')) as fp:
                fp.side_effect = IOError()
                self.assertRaises(SaltInvocationError, gpg.import_key,
                                  filename='/path/to/public-key-file')

            self.assertDictEqual(gpg.import_key(text='-BEGIN PGP PUBLIC KEY BLOCK-'), ret)

    # 'export_key' function tests: 1

    def test_export_key(self):
        '''
        Tests if it export a key from the GPG keychain
        '''
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertTrue(gpg.export_key(keyids='3F0C8E90D459D89A'))

    # 'receive_keys' function tests: 1

    def test_receive_keys(self):
        '''
        Tests if it receive key(s) from keyserver and add them to keychain
        '''
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user}):
            self.assertDictEqual(gpg.receive_keys(keys=['3F0C8E90D459D89A']),
                                 {'res': True,
                                  'message': ['Key F321F added to keychain'],
                                  'changes': {}})

    # 'trust_key' function tests: 1

    def test_trust_key(self):
        '''
        Tests if it set the trust level for a key in GPG keychain
        '''
        ret = {'message': 'Only specify one argument, fingerprint or keyid',
               'res': False}

        ret1 = {'message': 'KeyID 3F0C8 not in GPG keychain', 'res': False}

        ret2 = {'message': 'Required argument, fingerprint or keyid',
                'res': False}

        ret3 = ('ERROR: Valid trust levels - expired,unknown,'
                'not_trusted,marginally,fully,ultimately')

        ret4 = {'res': False,
                'message': 'Fingerprint not found for keyid 3F0C8E90D459D89A'}

        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_cmd = MagicMock(return_value={'retcode': 1, 'stderr': 'error'})
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'cmd.run_all': mock_cmd}):
            self.assertDictEqual(gpg.trust_key(keyid='3F0C8E90D459D89A',
                                               fingerprint='53C'), ret)

            self.assertDictEqual(gpg.trust_key(keyid='3F0C8'), ret1)

            self.assertDictEqual(gpg.trust_key(), ret2)

            self.assertEqual(gpg.trust_key(fingerprint='53C9'), ret3)

            self.assertEqual(gpg.trust_key(fingerprint='53C96',
                                           trust_level='not_trusted'),
                             {'res': False, 'message': 'error'})

        with patch.object(gpg, 'get_key', MagicMock(return_value=RET)):
            self.assertDictEqual(gpg.trust_key(keyid='3F0C8E90D459D89A'),
                                 ret4)

    # 'sign' function tests: 1

    def test_sign(self):
        '''
        Tests if it sign message or file
        '''
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_pillar = MagicMock(return_value=False)
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'pillar.item': mock_pillar}):
            self.assertRaises(SaltInvocationError, gpg.sign,
                              use_passphrase=True)

            self.assertRaises(SaltInvocationError, gpg.sign)

            self.assertTrue(gpg.sign(text='Hello there.  How are you?'))

    # 'verify' function tests: 1

    def test_verify(self):
        '''
        Tests if it verify a message or file
        '''
        ret = {'message': 'The signature could not be verified.', 'res': False}
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_pillar = MagicMock(return_value=False)
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'pillar.item': mock_pillar}):
            self.assertRaises(SaltInvocationError, gpg.verify)

            self.assertDictEqual(gpg.verify(text='Hello there.  How are you?'),
                                 ret)

    # 'encrypt' function tests: 1

    def test_encrypt(self):
        '''
        Tests if it encrypt a message or file
        '''
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_pillar = MagicMock(return_value=False)
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'pillar.item': mock_pillar}):
            self.assertRaises(SaltInvocationError, gpg.encrypt,
                              use_passphrase=True)

            self.assertRaises(SaltInvocationError, gpg.encrypt)

            self.assertDictEqual(gpg.encrypt(text='Hello there.  How are you?'),
                                 {'comment': True, 'res': True})

    # 'decrypt' function tests: 1

    def test_decrypt(self):
        '''
        Tests if it decrypt a message or file
        '''
        mock_conf = MagicMock(return_value='')
        mock_user = MagicMock(return_value={'home': 'salt'})
        mock_pillar = MagicMock(return_value=False)
        with patch.dict(gpg.__salt__, {'config.option': mock_conf,
                                       'user.info': mock_user,
                                       'pillar.item': mock_pillar}):
            self.assertRaises(SaltInvocationError, gpg.decrypt,
                              use_passphrase=True)

            self.assertRaises(SaltInvocationError, gpg.decrypt)

            self.assertDictEqual(gpg.decrypt(text='Hello there.  How are you?'),
                                 {'comment': True, 'res': True})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GpgTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Management of firewalld

.. versionadded:: Beryllium

The following example applies changes to the public zone, blocks echo-reply
and echo-request packets, does not set the zone to be the default, enables
masquerading, and allows ports 22/tcp and 25/tcp.

.. code-block:: yaml

    public:
      - name: public
      - block_icmp
        - echo-reply
        - echo-request
      - default: False
      - masquerade: True
      - ports:
        - 22/tcp
        - 25/tcp

The following example applies changes to the public zone, enables
masquerading and configures port forwarding TCP traffic from port 22
to 2222, and forwards TCP traffic from port 80 to 443 at 192.168.0.1.

.. code-block:: yaml

  my_zone:
    firewalld.present:
      - name: public
      - masquerade: True
      - port_fwd:
        - 22:2222:tcp
        - 80:443:tcp:192.168.0.1
'''
from __future__ import absolute_import

import logging
import salt.exceptions

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Ensure the firewall-cmd is available
    '''
    if salt.utils.which('firewall-cmd'):
        return True

    return False


def present(name,
            block_icmp=None,
            default=None,
            masquerade=False,
            ports=None,
            port_fwd=None,
            services=None):
    '''
    Ensure a zone has specific attributes
    '''
    ret = {'name': name,
           'result': True,
           'changes': {'icmp_blocks': [],
                       'ports': [],
                       'port_fwd': [],
                       'services': []},
           'comment': {'icmp_blocks': [],
                       'ports': [],
                       'port_fwd': [],
                       'services': []}}

    if name not in __salt__['firewalld.get_zones']():
        if __opts__['test']:
            ret['comment'][name] = '`{0}` will be created'.format(name)
        else:
            __salt__['firewalld.new_zone'](name)
            ret['changes'][name] = '`{0}` zone has been successfully created'.format(name)
    else:
        ret['comment'][name] = '`{0}` zone already exists'.format(name)

    if block_icmp:
        _valid_icmp_types = __salt__['firewalld.get_icmp_types']()
        _current_icmp_blocks = __salt__['firewalld.list_icmp_block'](name)

        for icmp_type in set(block_icmp):
            if icmp_type in _valid_icmp_types:
                if icmp_type in _current_icmp_blocks:
                    ret['comment']['icmp_blocks'].append(
                        '`{0}` already exists'.format(icmp_type)
                    )
                else:
                    if __opts__['test']:
                        ret['comment']['icmp_blocks'].append(
                            '`{0}` will be blocked'.format(icmp_type)
                        )
                    else:
                        __salt__['firewalld.block_icmp'](name, icmp_type)
                        ret['changes']['icmp_blocks'].append(
                            '`{0}` has been blocked'.format(icmp_type)
                        )
            else:
                log.error('{0} is an invalid ICMP type'.format(icmp_type))

    if default:
        if __salt__['firewalld.default_zone']() == name:
            ret['comment']['default'] = '`{0}` is already the default zone'.format(name)
        else:
            if __opts__['test']:
                ret['comment']['default'] = '`{0}` wll be set to the default zone'.format(name)
            else:
                __salt__['firewalld.set_default_zone'](name)
                ret['changes']['default'] = '`{0}` has been set to the default zone'.format(name)

    if masquerade:
        if __salt__['firewalld.get_masquerade'](name):
            ret['comment']['masquerade'] = 'masquerading is already enabed'
        else:
            if __opts__['test']:
                ret['comment']['masquerade'] = 'masquerading will be enabled'
            else:
                __salt__['firewalld.add_masquerade'](name)
                ret['changes']['masquerade'] = 'masquerading successfully set'

    if ports:
        _current_ports = __salt__['firewalld.list_ports'](name)

        for port in ports:
            if port in _current_ports:
                ret['comment']['ports'].append(
                    '`{0}` already exists'.format(port)
                )
            else:
                if __opts__['test']:
                    ret['comment']['ports'].append(
                        '{0} will be added'.format(port)
                    )
                else:
                    __salt__['firewalld.add_port'](name, port)
                    ret['changes']['ports'].append(
                        '`{0}` has been added to the firewall'.format(port)
                    )

    if port_fwd:
        _current_port_fwd = __salt__['firewalld.list_port_fwd'](name)

        for port in port_fwd:
            dstaddr = ''
            rule_exists = False

            if len(port.split(':')) > 3:
                (src, dest, protocol, dstaddr) = port.split(':')
            else:
                (src, dest, protocol) = port.split(':')

            for i in _current_port_fwd:
                if (src == i['Source port'] and dest == i['Destination port'] and
                        protocol == i['Protocol'] and dstaddr == i['Destination address']):
                    rule_exists = True

            if rule_exists:
                ret['comment']['port_fwd'].append(
                    '`{0}` port forwarding already exists'.format(port)
                )
            else:
                if __opts__['test']:
                    ret['comment']['port_fwd'].append(
                        '`{0}` port will be added'.format(port)
                    )
                else:
                    __salt__['firewalld.add_port_fwd'](
                        name, src, dest, protocol, dstaddr
                    )
                    ret['changes']['port_fwd'].append(
                        '`{0}` port forwarding has been added'.format(port)
                    )

    if services:
        _current_services = __salt__['firewalld.list_services'](name)

        for service in services:
            if service in _current_services:
                ret['comment']['services'].append(
                    '`{0}` service already exists'.format(service)
                )
            else:
                if __opts__['test']:
                    ret['comment']['services'].append(
                        '`{0}` service will be added'.format(service)
                    )
                else:
                    __salt__['firewalld.new_service'](service)
                    ret['changes']['services'].append(
                        '`{0}` has been successfully added'.format(service)
                    )

    return ret



# -*- coding: utf-8 -*-
'''
Return cached data from minions
'''
from __future__ import absolute_import
# Import python libs
import logging

# Import salt libs
import salt.log
import salt.utils
import salt.utils.master
import salt.payload
from salt.ext.six import string_types

log = logging.getLogger(__name__)


def grains(tgt=None, expr_form='glob', outputter=None, **kwargs):
    '''
    Return cached grains of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.grains
    '''
    deprecated_minion = kwargs.get('minion', None)
    if tgt is None and deprecated_minion is None:
        tgt = '*'  # targat all minions for backward compatibility
    elif tgt is None and isinstance(deprecated_minion, string_types):
        salt.utils.warn_until(
            'Boron',
            'The \'minion\' argument to the cache.grains runner is '
            'deprecated. Please specify the minion using the \'tgt\' '
            'argument.'
        )
        tgt = deprecated_minion
    elif tgt is None:
        return {}
    pillar_util = salt.utils.master.MasterPillarUtil(tgt, expr_form,
                                                     use_cached_grains=True,
                                                     grains_fallback=False,
                                                     opts=__opts__)
    cached_grains = pillar_util.get_minion_grains()
    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the cache.grains runner has '
            'been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': cached_grains}
    else:
        return cached_grains


def pillar(tgt=None, expr_form='glob', outputter=None, **kwargs):
    '''
    Return cached pillars of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.pillar
    '''
    deprecated_minion = kwargs.get('minion', None)
    if tgt is None and deprecated_minion is None:
        tgt = '*'  # targat all minions for backward compatibility
    elif tgt is None and isinstance(deprecated_minion, string_types):
        salt.utils.warn_until(
            'Boron',
            'The \'minion\' argument to the cache.pillar runner is '
            'deprecated. Please specify the minion using the \'tgt\' '
            'argument.'
        )
        tgt = deprecated_minion
    elif tgt is None:
        return {}
    pillar_util = salt.utils.master.MasterPillarUtil(tgt, expr_form,
                                                     use_cached_grains=True,
                                                     grains_fallback=False,
                                                     use_cached_pillar=True,
                                                     pillar_fallback=False,
                                                     opts=__opts__)
    cached_pillar = pillar_util.get_minion_pillar()
    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the cache.pillar runner has '
            'been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': cached_pillar}
    else:
        return cached_pillar


def mine(tgt=None, expr_form='glob', outputter=None, **kwargs):
    '''
    Return cached mine data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.mine
    '''
    deprecated_minion = kwargs.get('minion', None)
    if tgt is None and deprecated_minion is None:
        tgt = '*'  # targat all minions for backward compatibility
    elif tgt is None and isinstance(deprecated_minion, string_types):
        salt.utils.warn_until(
            'Boron',
            'The \'minion\' argument to the cache.mine runner is '
            'deprecated. Please specify the minion using the \'tgt\' '
            'argument.'
        )
        tgt = deprecated_minion
    elif tgt is None:
        return {}
    pillar_util = salt.utils.master.MasterPillarUtil(tgt, expr_form,
                                                     use_cached_grains=False,
                                                     grains_fallback=False,
                                                     use_cached_pillar=False,
                                                     pillar_fallback=False,
                                                     opts=__opts__)
    cached_mine = pillar_util.get_cached_mine_data()
    if outputter:
        salt.utils.warn_until(
            'Boron',
            'The \'outputter\' argument to the cache.mine runner has '
            'been deprecated. Please specify an outputter using --out. '
            'See the output of \'salt-run -h\' for more information.'
        )
        return {'outputter': outputter, 'data': cached_mine}
    else:
        return cached_mine


def _clear_cache(tgt=None,
                 expr_form='glob',
                 clear_pillar_flag=False,
                 clear_grains_flag=False,
                 clear_mine_flag=False,
                 clear_mine_func_flag=None):
    '''
    Clear the cached data/files for the targeted minions.
    '''
    if tgt is None:
        return False
    pillar_util = salt.utils.master.MasterPillarUtil(tgt, expr_form,
                                                     use_cached_grains=True,
                                                     grains_fallback=False,
                                                     use_cached_pillar=True,
                                                     pillar_fallback=False,
                                                     opts=__opts__)
    return pillar_util.clear_cached_minion_data(clear_pillar=clear_pillar_flag,
                                                clear_grains=clear_grains_flag,
                                                clear_mine=clear_mine_flag,
                                                clear_mine_func=clear_mine_func_flag)


def clear_pillar(tgt=None, expr_form='glob'):
    '''
    Clear the cached pillar data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.clear_pillar
    '''
    return _clear_cache(tgt, expr_form, clear_pillar_flag=True)


def clear_grains(tgt=None, expr_form='glob'):
    '''
    Clear the cached grains data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.clear_grains
    '''
    return _clear_cache(tgt, expr_form, clear_grains_flag=True)


def clear_mine(tgt=None, expr_form='glob'):
    '''
    Clear the cached mine data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.clear_mine
    '''
    return _clear_cache(tgt, expr_form, clear_mine_flag=True)


def clear_mine_func(tgt=None, expr_form='glob', clear_mine_func_flag=None):
    '''
    Clear the cached mine function data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.clear_mine_func tgt='*' clear_mine_func='network.interfaces'
    '''
    return _clear_cache(tgt, expr_form, clear_mine_func_flag=clear_mine_func_flag)


def clear_all(tgt=None, expr_form='glob'):
    '''
    Clear the cached pillar, grains, and mine data of the targeted minions

    CLI Example:

    .. code-block:: bash

        salt-run cache.clear_all
    '''
    return _clear_cache(tgt,
                        expr_form,
                        clear_pillar_flag=True,
                        clear_grains_flag=True,
                        clear_mine_flag=True)



# -*- coding: utf-8 -*-
'''
Send events covering service status
'''

# Import Python Libs
from __future__ import absolute_import

import os
import logging

log = logging.getLogger(__name__)  # pylint: disable=invalid-name

last_status = {}


def validate(config):
    '''
    Validate the beacon configuration
    '''
    # Configuration for service beacon should be a list of dicts
    if not isinstance(config, dict):
        log.info('Configuration for service beacon must be a dictionary.')
        return False
    return True


def beacon(config):
    '''
    Scan for the configured services and fire events

    Example Config

    .. code-block:: yaml

        beacons:
          service:
            salt-master:
            mysql:

    The config above sets up beacons to check for
    the salt-master and mysql services.

    The config also supports two other parameters for each service:

    `onchangeonly`: when `onchangeonly` is True the beacon will fire
    events only when the service status changes.  Otherwise, it will fire an event
    at each beacon interval.  The default is False.

    `uncleanshutdown`: If `uncleanshutdown` is present it should point to the location
    of a pid file for the service.  Most services will not clean up this pid file
    if they are shutdown uncleanly (e.g. via `kill -9`) or if they are terminated
    through a crash such as a segmentation fault.  If the file is present, then
    the beacon will add `uncleanshutdown: True` to the event.  If not present,
    the field will be False.  The field is only added when the service is NOT running.
    Omitting the configuration variable altogether will turn this feature off.

    Here is an example that will fire an event whenever the state of nginx changes
    and report an uncleanshutdown.  This example is for Arch, which places nginx's
    pid file in `/run`.

    .. code-block:: yaml

        beacons:
          service:
            nginx:
              onchangeonly: True
              uncleanshutdown: /run/nginx.pid
    '''
    ret = []
    for service in config:
        ret_dict = {}
        ret_dict[service] = {'running': __salt__['service.status'](service)}

        # We only want to report the nature of the shutdown
        # if the current running status is False
        # as well as if the config for the beacon asks for it
        if 'uncleanshutdown' in config[service] and not ret_dict[service]['running']:
            filename = config[service]['uncleanshutdown']
            if os.path.exists(filename):
                ret_dict[service]['shutdown'] = 'unclean'
            else:
                ret_dict[service]['shutdown'] = 'clean'

        if 'onchangeonly' in config[service] and config[service]['onchangeonly'] is True:
            if service not in last_status:
                last_status[service] = ''
            if last_status[service] != ret_dict[service]:
                last_status[service] = ret_dict[service]
                ret.append(ret_dict)
        else:
            ret.append(ret_dict)

    return ret



# -*- coding: utf-8 -*-
'''
Clone a remote SVN repository and use the filesystem as a Pillar source

This external Pillar source can be configured in the master config file like
so:

.. code-block:: yaml

    ext_pillar:
      - svn: trunk svn://svnserver/repo root=subdirectory

The `root=` parameter is optional and used to set the subdirectory from where
to look for Pillar files (such as ``top.sls``).

.. versionchanged:: 2014.7.0
    The optional ``root`` parameter will be added.

Note that this is not the same thing as configuring pillar data using the
:conf_master:`pillar_roots` parameter. The branch referenced in the
:conf_master:`ext_pillar` entry above (``master``), would evaluate to the
``base`` environment, so this branch needs to contain a ``top.sls`` with a
``base`` section in it, like this:

.. code-block:: yaml

    base:
      '*':
        - foo

To use other environments from the same SVN repo as svn_pillar sources, just
add additional lines, like so:

.. code-block:: yaml

    ext_pillar:
      - svn: trunk svn://svnserver/repo
      - svn: dev svn://svnserver/repo

In this case, the ``dev`` branch would need its own ``top.sls`` with a ``dev``
section in it, like this:

.. code-block:: yaml

    dev:
      '*':
        - bar
'''
from __future__ import absolute_import

# Import python libs
from copy import deepcopy
import logging
import os
import hashlib

# Import third party libs
HAS_SVN = False
try:
    import pysvn
    HAS_SVN = True
    CLIENT = pysvn.Client()
except ImportError:
    pass

# Import salt libs
from salt.pillar import Pillar

# Set up logging
log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'svn'


def __virtual__():
    ext_pillar_sources = [x for x in __opts__.get('ext_pillar', [])]
    if not any(['svn' in x for x in ext_pillar_sources]):
        return False
    if not HAS_SVN:
        log.error('SVN-based ext_pillar is enabled in configuration but '
                  'could not be loaded, is pysvn installed?')
        return False
    return __virtualname__


class SvnPillar(object):
    '''
    Deal with the remote SVN repository for Pillar
    '''

    def __init__(self, branch, repo_location, root, opts):
        '''
        Try to initialize the SVN repo object
        '''
        repo_hash = hashlib.md5(repo_location).hexdigest()
        repo_dir = os.path.join(opts['cachedir'], 'pillar_svnfs', repo_hash)

        self.branch = branch
        self.root = root
        self.repo_dir = repo_dir
        self.repo_location = repo_location

        if not os.path.isdir(repo_dir):
            os.makedirs(repo_dir)
            log.debug('Checking out fileserver for svn_pillar module')
            try:
                CLIENT.checkout(repo_location, repo_dir)
            except pysvn.ClientError:
                log.error('Failed to initialize svn_pillar {0} {1}'.format(repo_location, repo_dir))

    def update(self):
        try:
            log.debug('Updating fileserver for svn_pillar module')
            CLIENT.update(self.repo_dir)
        except pysvn.ClientError as exc:
            log.error('Unable to fetch the latest changes from remote '
                      '{0}: {1}'.format(self.repo_location, exc))

    def pillar_dir(self):
        '''
        Returns the directory of the pillars (repo cache + branch + root)
        '''
        repo_dir = self.repo_dir
        root = self.root
        branch = self.branch
        if branch == 'trunk' or branch == 'base':
            working_dir = os.path.join(repo_dir, 'trunk', root)
            if not os.path.isdir(working_dir):
                log.error('Could not find {0}/trunk/{1}'.format(self.repo_location, root))
            else:
                return os.path.normpath(working_dir)
        working_dir = os.path.join(repo_dir, 'branches', branch, root)
        if os.path.isdir(working_dir):
            return os.path.normpath(working_dir)
        working_dir = os.path.join(working_dir, 'tags', branch, root)
        if os.path.isdir(working_dir):
            return os.path.normpath(working_dir)
        log.error('Could not find {0}/branches/{1}/{2}'.format(self.repo_location, branch, root))
        return repo_dir


def _extract_key_val(kv, delimiter='='):
    '''Extract key and value from key=val string.

    Example:
    >>> _extract_key_val('foo=bar')
    ('foo', 'bar')
    '''
    pieces = kv.split(delimiter)
    key = pieces[0]
    val = delimiter.join(pieces[1:])
    return key, val


def ext_pillar(minion_id,
               pillar,  # pylint: disable=W0613
               repo_string):
    '''
    Execute a command and read the output as YAML
    '''
    # split the branch, repo name and optional extra (key=val) parameters.
    options = repo_string.strip().split()
    branch = options[0]
    repo_location = options[1]
    root = ''

    for extraopt in options[2:]:
        # Support multiple key=val attributes as custom parameters.
        DELIM = '='
        if DELIM not in extraopt:
            log.error('Incorrectly formatted extra parameter. '
                      'Missing {0!r}: {1}'.format(DELIM, extraopt))
        key, val = _extract_key_val(extraopt, DELIM)
        if key == 'root':
            root = val
        else:
            log.warning('Unrecognized extra parameter: {0}'.format(key))

    svnpil = SvnPillar(branch, repo_location, root, __opts__)

    # environment is "different" from the branch
    branch = (branch == 'trunk' and 'base' or branch)

    pillar_dir = svnpil.pillar_dir()
    log.debug("[pillar_roots][{0}] = {1}".format(branch, pillar_dir))

    # Don't recurse forever-- the Pillar object will re-call the ext_pillar
    # function
    if __opts__['pillar_roots'].get(branch, []) == [pillar_dir]:
        return {}
    svnpil.update()
    opts = deepcopy(__opts__)
    opts['pillar_roots'][branch] = [pillar_dir]
    pil = Pillar(opts, __grains__, minion_id, branch)
    return pil.compile_pillar()
